import logging

import wx

import pypatterns.command as CommandModule
import pypatterns.filter as FilterModule
import pypatterns.relational as RelationalModule
import pypatterns.relational.commands as RelationalCommandModule

import pomsets_app.controller.automaton as AutomatonModule
import pomsets_app.controller.commands as CloudCommandModule

import pomsets_app.gui.commands as GuiCommandModule
import pomsets_app.gui.grid as PomsetGUITableModule

import pomsets_app.gui.widget as WidgetModule

ID_DIALOG_VIRTUALMACHINES = wx.NewId()


class ConfigDialog(WidgetModule.Dialog):

    XRC_NAME = 'EC2Dialog'
    WIDGET_BINDINGS = WidgetModule.Dialog.WIDGET_BINDINGS + []

    def __init__(self, xmlResource, parent=None):
        WidgetModule.Dialog.__init__(
            self, xmlResource.LoadDialog(parent, ConfigDialog.XRC_NAME))

        valueKeyPair = self.getWidget('ValueKeyPair')
        valueKeyPair.SetMinSize(wx.Size(300, 25))
        valueIdentityFile = self.getWidget('ValueIdentityFile')
        valueIdentityFile.SetMinSize(wx.Size(300, 30))

        self.SetMinSize(wx.Size(325, 150))
        self.Fit()
        self.SetAutoLayout(True)
        return

    pass


class VirtualMachineWindow(wx.Frame):

    def __init__(self, parent, automaton, id, title, dataModel):
        wx.Frame.__init__(self, parent, id, 
                          size=(650, 320), name=title)


        panel = wx.ScrolledWindow(self, -1)
        panel.EnableScrolling(True,True)

        vbox = wx.BoxSizer(wx.VERTICAL)

        table = PomsetGUITableModule.Table(
            dataModel, automaton.contextManager().COLUMNS_VIRTUALMACHINE_DISPLAY)

        self.grid = wx.grid.Grid(panel, size=(640,250))
        self.grid.SetTable(table)
        self.grid.EnableEditing(False)
        self.grid.SetSelectionMode(wx.grid.Grid.wxGridSelectCells)

        vbox.Add(panel)


        hbox = wx.BoxSizer(wx.HORIZONTAL)

        credentialsButton = wx.Button(self, -1, 'Credentials', size=(120, 30))
        assignButton = wx.Button(self, -1, 'Assign', size=(120, 30))
        unassignButton = wx.Button(self, -1, 'Unassign', size=(120, 30))
        refreshButton = wx.Button(self, -1, 'Refresh', size=(70, 30))
        closeButton = wx.Button(self, -1, 'Close', size=(70, 30))

        hbox.Add(credentialsButton, 1)
        hbox.Add(assignButton, 1, wx.LEFT, 5)
        hbox.Add(unassignButton, 1, wx.LEFT, 5)
        hbox.Add(refreshButton, 1, wx.LEFT, 5)
        hbox.Add(closeButton, 1, wx.LEFT, 5)

        credentialsButton.Bind(wx.EVT_BUTTON, self.OnButtonCredentials)
        assignButton.Bind(wx.EVT_BUTTON, self.OnButtonAssign)
        unassignButton.Bind(wx.EVT_BUTTON, self.OnButtonUnassign)
        refreshButton.Bind(wx.EVT_BUTTON, self.OnButtonRefresh)
        closeButton.Bind(wx.EVT_BUTTON, self.OnButtonClose)

        vbox.Add(hbox, 1)

        self.SetSizer(vbox)


        # TODO:
        # add as listener to the system's events
        # generally modifications to this table
        # will only come from commands generated here
        # except when the workers take on tasks from the queue
        # so will need to listen to those events
        # in order to update the data


        return

    def OnButtonCredentials(self, event):

        import pomsets_app.gui.utils as GuiModule
        GuiModule.manageEc2Credentials(self.contextManager, self.GetParent())
        pass


    '''
    def OnButtonAdd(self, event):
        """
        This function as been deprecated
        until we decide to re-incorporate de/provisioning abilities
        back on the system
        """
        contextManager = self.contextManager
        automaton = contextManager.automaton()

        # TODO:
        # should bring up an option for the user to select
        # a particular service name
        # and not hardcode 'eucalyptus'

        command = CommandModule.CompositeCommand()

        # create the provision command
        provisionVmCommand = CloudCommandModule.ProvisionVmCommand(
            automaton, 'eucalyptus', keyPair, imageId)
        command.addCommand(provisionVmCommand)

        # add the commandbuilder
        # which will add the command to assign a worker to the vm
        commandBuilder = \
            CloudCommandModule.AssignWorkerToVmCommandBuilder(
            provisionVmCommand
            )
        command.addCommandBuilder(provisionVmCommand, commandBuilder)

        # add the commandbuilder
        # which will add the commands to synchronize the relational table
        commandBuilder = \
            CloudCommandModule.AddNewEntryToVmTableCommandBuilder(
            provisionVmCommand
            )
        command.addCommandBuilder(provisionVmCommand, commandBuilder)

        # now add the gui synchronization command
        command.addCommandBuilder(
            provisionVmCommand, 
            GuiCommandModule.NotifyRowAddedCommandBuilder(
                provisionVmCommand, self.grid)
        )

        automaton.executeCommand(command)

        pass
    '''

    def OnButtonAssign(self, event):

        contextManager = self.contextManager
        automaton = contextManager.automaton()


        selectedVmInstances = self.getSelectedVmInstances()

        dataModel = self.grid.GetTable().dataModel()

        command = CommandModule.CompositeCommand()
        for vmInstance in selectedVmInstances:

            executeEnvironments = \
                automaton.getExecuteEnvironmentsFromVmInstance(vmInstance)
            if len(executeEnvironments) is not 0:
                continue
            serviceAPI = automaton.getServiceApiFromVmInstance(vmInstance)

            assignEnvCommand = \
                CloudCommandModule.AssignExecuteEnvironmentForVmCommand(
                automaton, serviceAPI, vmInstance)

            command.addCommand(assignEnvCommand)
            pass


        executeEnvironments = []
        if len(command._subcommands):

            # execute the command
            success = automaton.executeCommand(command)

            # if not successful
            if not success:
                logging.error('failed creating execute environments for vmInstances %s' % selectedVmInstances)
                return
            pass
        else:
            logging.error('no commands were generated for assigning execute environments')

        executeEnvironments = {}
        for vmInstance in selectedVmInstances:

            #executeEnvironments.extend(
            #    automaton.getExecuteEnvironmentsFromVmInstance(vmInstance))
            executeEnvironments[vmInstance] = \
                automaton.getExecuteEnvironmentsFromVmInstance(vmInstance)
            pass


        if len(executeEnvironments) is 0:
            logging.error('cannot assign threads as no execute environments are selected')
            return


        command = CommandModule.CompositeCommand()

        # for executeEnvironment in executeEnvironments:
        for vmInstance, executeEnvironmentList in executeEnvironments.iteritems():

            if len(executeEnvironmentList) is not 1:
                raise NotImplementedError('need to handle multiple execute environments for the same vm instance')
            executeEnvironment = executeEnvironmentList[0]

            threads = automaton.getThreadsFromExecuteEnvironment(executeEnvironment)
            if len(threads) is not 0:
                # TODO:
                # will need a way to create multiple threads
                # for the same execute environment
                continue

            # TODO:
            # add ability to parameterize the threadPoolId
            threadPoolId = AutomatonModule.ID_EXECUTE_EUCA2OOLS

            assignWorkerCommand = \
                CloudCommandModule.AssignThreadForExecuteEnvironmentCommand(
                automaton, threadPoolId, executeEnvironment
                )

            command.addCommand(assignWorkerCommand)


            # cannot set thread
            # because it is not getting displayed
            #command.addCommandBuilder(
            #    assignWorkerCommand,
            #    GuiCommandModule.SetVmThreadCommandBuilder(
            #        assignWorkerCommand, self.grid, vmInstance
            #    )
            #)


            rowFilter = RelationalModule.ColumnValueFilter(
                'Instance ID',
                FilterModule.EquivalenceFilter(vmInstance))
            # since we expecte the command to succeed
            # we add a command that will set the value of "Enabled" to True
            command.addCommandBuilder(
                assignWorkerCommand,
                GuiCommandModule.UpdateGridCellValueCommandBuilder(
                    assignWorkerCommand, self.grid, 
                    rowFilter, 'Enabled', 'True'))
            pass


        success = automaton.executeCommand(command)

        # TODO: 
        # something needs to update the grid

        # update the execute menu items
        contextManager.app().GetTopWindow().updateExecuteMenus()

        return


    def OnButtonRefresh(self, event):

        from euca2ools import Euca2ool, InstanceValidationError, Util

        contextManager = self.contextManager
        automaton = contextManager.automaton()

        # TODO:
        # need to check if we have the necessary credentials
        # if not, query for it


        # TODO
        # retrieve all the currently active threads
        # associate the threads with the vmInstaces

        # first clear the table
        dataModel = self.grid.GetTable().dataModel()
        rows = dataModel.clear()

        vmEnvs = {}
        envThreads = {}
        for row in rows:
            vmInstance = row.getColumn('Instance ID')
            executeEnvironments = automaton.getExecuteEnvironmentsFromVmInstance(vmInstance)
            if len(executeEnvironments) is 0:
                continue

            vmEnvs[vmInstance] = executeEnvironments[0]
            threads = automaton.getThreadsFromExecuteEnvironment(executeEnvironments[0])
            if len(threads) is 0:
                continue

            envThreads[executeEnvironments[0]] = threads[0]

            # TODO:
            # need to handle assigned tasks

            pass

        if len(rows):
            msg = wx.grid.GridTableMessage(
                self.grid.GetTable(),
                wx.grid.GRIDTABLE_NOTIFY_ROWS_DELETED,
                len(rows)
            )
            self.grid.ProcessTableMessage(msg)

        reservations = []

        try:
            controllerValues = automaton.getCloudControllerCredentialsForAPI('euca2ools')
        except KeyError, e:
            print "should query user for credentials"
            pass


        try:

            # 'Service', 'Reservation ID', 'Instance ID', 'Image ID', 'IP', 
            # 'Enabled', 'Thread', 'Assigned task'

            euca = Euca2ool()

            euca.ec2_url = controllerValues['url']
            euca.ec2_user_access_key = controllerValues['access key']
            euca.ec2_user_secret_key = controllerValues['secret key']

            """
            # TODO:
            # can remove this once euca2ools uses an exception model
            # instead of just calling sys.exit
            if not os.getenv('EC2_ACCESS_KEY'):
                raise KeyError('no value for EC2_ACCESS_KEY specified')

            # TODO:
            # can remove this once euca2ools uses an exception model
            # instead of just calling sys.exit
            if not os.getenv('EC2_SECRET_KEY'):
                raise KeyError('no value for EC2_SECRET_KEY specified')

            self.ec2_url
            """

            euca_conn = euca.make_connection()

            reservations = euca_conn.get_all_instances([])
        except Exception, e:
            print "error on making connecting to Eucalyptus >> %s" % e
            pass

        print "after querying Eucalyptus"

        if len(reservations) is 0:
            print "no active reservations"

        rows = []
        for reservation in reservations:
            for instance in reservation.instances:
                # reservation.id
                # instance.id
                # instance.image_id
                # instance.public_dns_name
                # instance.state (this is so we dont show terminated ones)
                # instance.key_name
                row = dataModel.addRow()
                row.setColumn('Service', 'Eucalyptus')
                row.setColumn('API', 'euca2ools')
                row.setColumn('Reservation ID', reservation.id)
                row.setColumn('Instance ID', instance.id)
                row.setColumn('Image ID', instance.image_id)
                row.setColumn('IP', instance.public_dns_name)
                row.setColumn('Key name', instance.key_name)


                # TODO:
                # this should be a join operation


                vmEnv = vmEnvs.get(instance.id, None)
                envThread = None
                if vmEnv:
                    envThread = envThreads.get(vmEnv, None)
                    row.setColumn('Enabled', 'True')
                else:
                    row.setColumn('Enabled', 'False')
                

                rows.append(row)
                pass
            pass

        if len(rows):
            msg = wx.grid.GridTableMessage(
                self.grid.GetTable(),
                wx.grid.GRIDTABLE_NOTIFY_ROWS_APPENDED,
                len(rows)
            )
            self.grid.ProcessTableMessage(msg)
            pass


        event.Skip()
        return



    def OnButtonClose(self, event):

        # remove self from the list of listeners
        # otherwise this window will not 
        # be released, due to reference counting

        self.Close()
        pass


    def getSelectedVmInstances(self):
        # get the thread to be dismissed
        # get the vm to be terminated
        # (only if there are no other threads)

        vmInstances = []

        selectedRowIndices = self.grid.GetSelectedRows()
        for rowIndex in selectedRowIndices:
            columnIndex = self.grid.GetTable().getIndexForColumn('Instance ID')
            vmInstance = self.grid.GetTable().GetValue(rowIndex, columnIndex)
            vmInstances.append(vmInstance)

        return vmInstances


    def OnButtonUnassign(self, event):

        contextManager = self.contextManager
        automaton = contextManager.automaton()


        selectedVmInstances = self.getSelectedVmInstances()

        vmInstances = []

        dataModel = self.grid.GetTable().dataModel()
        for vmInstance in selectedVmInstances:
            filter = FilterModule.constructAndFilter()
            filter.addFilter(
                RelationalModule.ColumnValueFilter(
                    'Instance ID',
                    FilterModule.EquivalenceFilter(vmInstance)
                    )
                )
            filter.addFilter(
                RelationalModule.ColumnValueFilter(
                    'Enabled',
                    FilterModule.EquivalenceFilter('True'))
                )
            #rows = [x for x in dataModel.retrieve(filter, ['Thread'])]
            #thread = rows[0][0]
            #if not (thread is RelationalModule.Table.NULL or
            #        thread is None):
            #    vmInstances.append(vmInstance)
            rows = [x for x in dataModel.retrieve(filter)]
            if len(rows):
                vmInstances.append(vmInstance)
            pass


        if len(vmInstances) is 0:
            print "nothing to do, as no rows are selected"
            return

        command = CommandModule.CompositeCommand()

        # TODO:
        # add ability to parameterize the threadPoolId
        threadPoolId = AutomatonModule.ID_EXECUTE_EUCA2OOLS

        for vmInstance in vmInstances:
            # create the command to dismiss the worker
            # but do it only if there's a worker assigned to the vm
            # which there should be
            dismissWorkerCommand = CloudCommandModule.UnassignWorkerFromVmCommand(
                automaton, threadPoolId, vmInstance
                )
            command.addCommand(dismissWorkerCommand)


            """
            command.addCommandBuilder(
                dismissWorkerCommand,
                GuiCommandModule.UnsetVmThreadCommandBuilder(
                    dismissWorkerCommand, self.grid
                )
            )
            """

            rowFilter = RelationalModule.ColumnValueFilter(
                'Instance ID',
                FilterModule.EquivalenceFilter(vmInstance))
            # since we expecte the command to succeed
            # we add a command that will set the value of "Enabled" to True
            command.addCommandBuilder(
                dismissWorkerCommand,
                GuiCommandModule.UpdateGridCellValueCommandBuilder(
                    dismissWorkerCommand, self.grid, 
                    rowFilter, 'Enabled', 'False'))

            pass

        automaton.executeCommand(command)

        return


    '''
    def OnButtonRemove(self, event):
        """
        this function is deprecated until we implement
        the ability to control clusters from within the application
        """

        # get the thread to be dismissed
        # get the vm to be terminated
        # (only if there are no other threads)
        selectedRowIndices = self.grid.GetSelectedRows()
        if len(selectedRowIndices) is 0:
            print "nothing to do, as no rows are selected"
            return
        elif len(selectedRowIndices) is not 1:
            raise NotImplementedError('currently implemented for only one row')

        columnIndex = self.grid.GetTable().getIndexForColumn('Instance ID')
        vmInstance = self.grid.GetTable().GetValue(selectedRowIndices[0], columnIndex)

        contextManager = self.contextManager
        automaton = contextManager.automaton()

        command = CommandModule.CompositeCommand()

        # create the command to dismiss the worker
        # but do it only if there's a worker assigned to the vm
        # which there should be
        dismissWorkerCommand = CloudCommandModule.UnassignWorkerFromVmCommand(
            automaton, vmInstance
            )
        command.addCommand(dismissWorkerCommand)

        # create the terminate command
        terminateVmCommand = CloudCommandModule.TerminateVmCommand(
            automaton, vmInstance)
        command.addCommand(terminateVmCommand)

        # create the command to remove entry from the table
        # unlike add, we do not use a command builder
        # because we already have all the data necessary
        filter = RelationalModule.ColumnValueFilter(
            'Instance ID',
            FilterModule.EquivalenceFilter(vmInstance.id)
        )
        removeEntryCommand = RelationalCommandModule.RemoveRowsCommand(
            automaton.virtualMachineTable(), filter)
        command.addCommand(removeEntryCommand)

        # TODO:
        # attach the command builder that creates the command
        # to notify that the grid needs to refresh

        automaton.executeCommand(command)

        return
    '''

    def populateData(self):
        return

    # END class VirtualMachineWindow
    pass



import currypy
import logging
import os
import sys
import threadpool
import time

import StringIO

import wx
import wx.lib.newevent
import wx.xrc

import numpy

import zgl_graphdrawer.Event as EventModule
import zgl_graphdrawer.Node as NodeModule

import pypatterns.filter as FilterModule

import pomsets.command as CommandModule
import pomsets.context as ContextModule
import pomsets.parameter as ParameterModule
import pomsets.task as TaskModule

import cloudpool.environment as ExecuteEnvironmentModule
import cloudpool.shell as ShellModule

import pypatterns.relational as RelationalModule

# Custom event classes
# http://wiki.wxpython.org/CustomEventClasses
PomsetCreatedEvent, EVT_POMSET_CREATED = wx.lib.newevent.NewEvent()
PomsetLoadedEvent, EVT_POMSET_LOADED = wx.lib.newevent.NewEvent()
PomsetClosedEvent, EVT_POMSET_CLOSED = wx.lib.newevent.NewEvent()
PomsetSavedEvent, EVT_POMSET_SAVED = wx.lib.newevent.NewEvent()
NodeCreatedEvent, EVT_NODE_CREATED = wx.lib.newevent.NewEvent()
NodeRemovedEvent, EVT_NODE_REMOVED = wx.lib.newevent.NewEvent()
EdgeCreatedEvent, EVT_EDGE_CREATED = wx.lib.newevent.NewEvent()
EdgeRemovedEvent, EVT_EDGE_REMOVED = wx.lib.newevent.NewEvent()

CommandExecutionFailedEvent, EVT_COMMAND_EXECUTION_FAILED = wx.lib.newevent.NewEvent()

NodeRenamedEvent, EVT_NODE_RENAMED = wx.lib.newevent.NewEvent()
DefinitionValuesChangedEvent, EVT_DEFINITION_VALUES_CHANGED = wx.lib.newevent.NewEvent()
TaskValuesChangedEvent, EVT_TASK_VALUES_CHANGED = wx.lib.newevent.NewEvent()
ParameterModifiedEvent, EVT_PARAMETER_MODIFIED = wx.lib.newevent.NewEvent()
ParameterCreatedEvent, EVT_PARAMETER_CREATED = wx.lib.newevent.NewEvent()
ParameterDeletedEvent, EVT_PARAMETER_DELETED = wx.lib.newevent.NewEvent()

TaskStatusChangedEvent, EVT_TASK_STATUS_CHANGED = wx.lib.newevent.NewEvent()

class CanvasEventHandler(EventModule.CanvasEventHandler):

    def bindEvents(self):
        self.Bind(EVT_POMSET_CREATED, self.OnPomsetCreated)
        self.Bind(EVT_POMSET_LOADED, self.OnPomsetLoaded)
        self.Bind(EVT_POMSET_CLOSED, self.OnPomsetClosed)
        self.Bind(EVT_POMSET_SAVED, self.OnPomsetSaved)
        self.Bind(EVT_NODE_CREATED, self.OnNodeCreated)
        self.Bind(EVT_NODE_RENAMED, self.OnNodeRenamed)
        self.Bind(EVT_NODE_REMOVED, self.OnNodeRemoved)
        self.Bind(EVT_EDGE_CREATED, self.OnEdgeCreated)
        self.Bind(EVT_EDGE_REMOVED, self.OnEdgeRemoved)

        self.Bind(EVT_DEFINITION_VALUES_CHANGED, self.OnDefinitionValuesChanged)
        self.Bind(EVT_TASK_VALUES_CHANGED, self.OnTaskValuesChanged)
        self.Bind(EVT_PARAMETER_MODIFIED, self.OnParameterModified)
        self.Bind(EVT_PARAMETER_CREATED, self.OnParameterCreated)
        self.Bind(EVT_PARAMETER_DELETED, self.OnParameterDeleted)

        self.Bind(EVT_TASK_STATUS_CHANGED, self.OnTaskStatusChanged)
        
        EventModule.CanvasEventHandler.bindEvents(self)
        return


    def OnPomsetSaved(self, event):

        contextManager = event.contextManager

        app = contextManager.app()
        frame = app.GetTopWindow()

        frame.populateActivePomsetTreeCtrl()        
        return


    def OnPomsetCreated(self, event):

        contextManager = event.contextManager
        app = contextManager.app()
        frame = app.GetTopWindow()

        pomsetContext = event.object

        contextManager.addActivePomsetContext(pomsetContext)

        contextManager.activePomsetContext(pomsetContext)
        frame.displayPomset(pomsetContext)

        return

    
    def OnPomsetClosed(self, event):

        canvas = self.eventSource()
        pomset = event.object
        contextManager = event.contextManager

        # TODO:
        # this should be an event
        canvas.resetDrawables()

        app = contextManager.app()
        frame = app.GetTopWindow()

        activeContext = contextManager.activePomsetContext()
        frame.displayPomset(activeContext)

        frame.populateActivePomsetTreeCtrl()        
        
        return

    
    def OnPomsetLoaded(self, event):
        
        canvas = self.eventSource()
        pomsetContext = event.object
        contextManager = event.contextManager
        path = event.path

        contextManager.addActivePomsetContext(pomsetContext)
        contextManager.activePomsetContext(pomsetContext)
        
        app = contextManager.app()
        frame = app.GetTopWindow()
        frame.displayPomset(pomsetContext)
        
        return


    def OnNodeCreated(self, event):
        
        # TODO:
        # do we need to ensure that the node
        # being is in the current active pomset?
        
        canvas = self.eventSource()

        dataNode = event.object


        uiNode = canvas.addNode(dataNode)
        uiNodeSize = uiNode.getBounds()


        # TODO:
        # consolidate code for setting
        # node position using canvas position
        
        guiOptions = dataNode.guiOptions()
        if hasattr(event, 'position'):
            position = event.position
            canvasMatrix = numpy.matrix(
                '1 0 0 0; 0 1 0 0; 0 0 1 0; %s %s 0 1' % 
                (position[0] - uiNodeSize[0]/2.0,
                 position[1] + uiNodeSize[1]/2.0)
            )
            worldMatrix = canvas.getWorldCoordinatesFromCanvasCoordinates(
                canvasMatrix)

            position = (worldMatrix[3,0], worldMatrix[3,1])

            guiOptions['canvas position'] = position
            pass
        else:
            # TODO:
            # this should be at the center of the canvas
            # instead of hardcoded to (0, 0)
            position = guiOptions.get('canvas position', (0, 0))

        uiNode.setPosition(*position)
        

        contextManager = event.contextManager
        contextManager.activePomsetContext().isModified(True)
        
        # now force the frame to move
        # so that the canvas will resize correctly
        app = contextManager.app()
        frame = app.GetTopWindow()
        # frame.resizeCanvas()

        frame.populateActivePomsetTreeCtrl()

        return



    def OnEdgeCreated(self, event):

        # TODO:
        # extract this into a function that can add 
        # an arbitrary edge into the canvas
        canvas = self.eventSource()
        path = event.path
        canvas.addEdge(path)
        return


    def OnEdgeRemoved(self, event):
        canvas = self.eventSource()
        edge = canvas.getGuiObjectForDataObject(event.object)
        canvas.removeDrawable(edge)
        return


    def OnDefinitionValuesChanged(self, event):
        """
        If the parameter ordering got changed
        the we may need to re-order the parameters in the canvas
        """
        print "%s OnDefinitionValuesChanged" % self.__class__
        return


    
    
    def OnTaskStatusChanged(self, event):
        # update the status
        event.node.updateExecuteStatus(event.status)
        # rebuild primitives
        event.node._children = event.node._buildChildren()
        return

    
    def OnTaskValuesChanged(self, event):
        print "%s OnTaskValuesChanged" % self.__class__
        return
    

    def OnParameterCreated(self, event):
        print "%s OnParameterCreated" % self.__class__
        return

    def OnParameterDeleted(self, event):
        print "%s OnParameterDeleted" % self.__class__
        return

    def OnParameterModified(self, event):
        # need to check whether a node 
        definition = event.definition

        generator = TaskModule.getTaskGeneratorForDefinition(definition)
        isParameterSweep = isinstance(
            generator, TaskModule.ParameterSweepTaskGenerator)

        canvas = self.eventSource()
        node = canvas.getGuiObjectForDataNode(definition)
        node.setImageParameterSweep(isParameterSweep)        
        return


    def OnNodeRemoved(self, event):
        canvas = self.eventSource()
        node = canvas.getGuiObjectForDataNode(event.object)
        canvas.removeDrawable(node)
        return


    def OnNodeRenamed(self, event):
        # rebuild primitives

        canvas = self.eventSource()
        node = canvas.getGuiObjectForDataNode(event.node)

        nameLabel = node.createNamePrimitive(canvas.nodePolicy())
        node.parts[NodeModule.Node.PART_LABEL] = nameLabel

        node._children = node._buildChildren()

        # if a node got renamed,
        # then the pomset context is modified
        contextManager = canvas.contextManager()
        contextManager.activePomsetContext().isModified(True)
        app = contextManager.app()
        frame = app.GetTopWindow()
        frame.populateActivePomsetTreeCtrl()

        return
    
    def OnPomsetEvent(self, event):
        print "need to respond to pomset event"
        return


    # END class CanvasEventHandler
    pass



class MenuEventHandler(wx.EvtHandler, EventModule.EventHandler):

    def __init__(self, eventSource):
        wx.EvtHandler.__init__(self)
        self.eventSource(eventSource)
        return

    def getApp(self):
        return self.eventSource().app()

    def bindEvents(self):
        import pomsets_app.gui.menu as MenuModule
        import pomsets_app.controller.automaton as AutomatonModule

        # file
        wx.EVT_MENU(self, wx.ID_NEW, self.OnCreateNewPomset)
        wx.EVT_MENU(self, wx.ID_OPEN, self.OnLoadExistingPomset)
        wx.EVT_MENU(self, wx.ID_SAVE, self.OnSaveCurrentPomset)
        wx.EVT_MENU(self, wx.ID_SAVEAS, self.OnSaveCurrentPomsetAs)
        wx.EVT_MENU(self, wx.ID_CLOSE, self.OnCloseCurrentPomset)

        #wx.EVT_MENU(self, MenuModule.ID_DEFINITIONS_EDIT, 
        #            self.OnEditDefinitionList)
        # wx.EVT_MENU(self, MenuModule.ID_NODE_ADD, self.OnAddNode)
        #wx.EVT_MENU(self, MenuModule.ID_NODE_REMOVE, self.OnRemoveNodes)

        # execute
        wx.EVT_MENU(self, AutomatonModule.ID_EXECUTE_LOCAL, self.OnExecutePomset)
        wx.EVT_MENU(self, AutomatonModule.ID_EXECUTE_REMOTE, self.OnExecutePomset)
        wx.EVT_MENU(self, AutomatonModule.ID_EXECUTE_EUCA2OOLS, self.OnExecutePomset)
        wx.EVT_MENU(self, AutomatonModule.ID_EXECUTE_PREVIEW, 
                    self.OnPreviewExecution)

        # connections
        #wx.EVT_MENU(self, MenuModule.ID_CONNECTION_MANAGE, self.OnManageConnections)
        #wx.EVT_MENU(self, MenuModule.ID_CONNECTION_CONNECT, self.OnConnect)
        #wx.EVT_MENU(self, MenuModule.ID_CONNECTION_DISCONNECT, self.OnDisconnect)
        wx.EVT_MENU(self, MenuModule.ID_ACCOUNT_MANAGE, self.OnDisplayVirtualMachines)
        wx.EVT_MENU(self, MenuModule.ID_HADOOP_MANAGE, self.OnManageHadoop)
        wx.EVT_MENU(self, MenuModule.ID_EUCA2OOLS_MANAGE, 
                    self.OnManageEC2Credentials)


        wx.EVT_MENU(self, wx.ID_ABOUT, self.OnAboutApplication)
        # wx.EVT_MENU(self, wx.ID_EXIT, self.OnExitApplication)
        return

    def OnUnknownAction(self, event):
        print "unknown action"
        return

    def OnMenuItem(self, event):
        menuId = event.GetMenuId()
        actionId = MenuEventHandler.MENU_EVENTS.get(menuId, 'OnUnknownAction')
        action = getattr(self, actionId)
        return action(event)

    def OnPreviewExecution(self, event):

        contextManager = self.getApp().contextManager()

        parent = self.eventSource()
        #res =  wx.xrc.XmlResource(
        #    '%s/app/resources/xrc/execution preview dialog.xrc' % os.getenv('APP_ROOT'))
        resourcePath = contextManager.resourcePath()
        res =  wx.xrc.XmlResource(
            os.path.join(resourcePath, 'xrc', 'execution preview dialog.xrc')
            )


        import pomsets_app.gui.widget.task as WidgetModule
        dialog = WidgetModule.ExecutionPreviewDialog(res, parent)

        stream = StringIO.StringIO()

        contextManager.outputCommandsToStream(stream)
        dialog.populateTextCtrl(stream.getvalue())
        # TODO:
        # should ensure that we wait only 
        # for the specific thread that's running the preview
        # and not for threads running other pomsets
        # in the case where we have multiple active pomsets
        # contextManager.automaton().threadpool().wait()

        # TODO: 
        # should provide the above with a callback
        # that will update the textCtrl
        # when the execution/printing has completed


        # event is the event originating from the contextual menu
        # self.event is the event
        # dialog.populateTextCtrl(value)
        try:
            dialog.ShowModal()
        finally:
            dialog.Destroy()

        return


    def OnDisplayVirtualMachines(self, event):

        contextManager = self.getApp().contextManager()

        automaton = contextManager.automaton()

        dataModel = automaton.virtualMachineTable()

        parent = self.eventSource()

        
        import pomsets_app.gui.widget.eucalyptus as WidgetModule
        vmWindow = WidgetModule.VirtualMachineWindow(
            parent,
            automaton,
            WidgetModule.ID_DIALOG_VIRTUALMACHINES,
            "Virtual machines",
            dataModel
        )
        # event is the event originating from the contextual menu
        # self.event is the event
        vmWindow.contextManager = contextManager
        vmWindow.populateData()

        vmWindow.Show()
        return


    def OnManageHadoop(self, event):
        contextManager = self.getApp().contextManager()

        parent = self.eventSource()
        import pomsets_app.gui.utils as GuiUtilsModule
        GuiUtilsModule.manageHadoopConfiguration(contextManager, parent)

        return

    def OnCreateNewDefinition(self, event):
        raise NotImplementedError('not implemented OnCreateNewDefinition')



    def OnEditDefinitionList(self, event):
        contextManager = self.getApp().contextManager()
        parent = self.eventSource()

        import pomsets_app.gui.utils as GuiUtilsModule
        GuiUtilsModule.editDefinitionList(contextManager, parent)
        return


    def OnAddNode(self, event):
        raise NotImplementedError('not implemented OnAddNode')

    def OnRemoveNodes(self, event):
        raise NotImplementedError('not implemented self.OnRemoveNodes')


    def OnManageEC2Credentials(self, event):

        contextManager = self.getApp().contextManager()
        parent = self.eventSource()


        import pomsets_app.gui.utils as GuiUtilsModule
        GuiUtilsModule.manageEc2Credentials(contextManager, parent)

        return


    def OnExecutePomset(self, event):

        import menu as MenuModule

        app = self.getApp()
        contextManager = app.contextManager()

        executionEnvId = event.GetId()

        try:
            import pomsets.command as ExecuteCommandModule
            commandBuilder = ExecuteCommandModule.CommandBuilder(
                ExecuteCommandModule.buildCommandFunction_commandlineArgs
            )
            commandBuilderMap = {
                'shell process':commandBuilder
            }

            # TODO:
            # is there a way to figure out env here,
            # rather than using executionEnvId
            
            requestKwds = contextManager.generateRequestKwds(
                executeEnvironmentId = executionEnvId,
                commandBuilderMap = commandBuilderMap)


            # TODO:
            # need to determine 

            contextManager.executePomset(
                pomset=contextManager.activePomset(), 
                requestKwds=requestKwds)

        except Exception, e:
            logging.error("error executing pomset >> %s" % e)
            
            # post an event about the failure
            failEvent = CommandExecutionFailedEvent(
                object=contextManager.activePomset(),
                exception=e,
                originalEvent=event)
            contextManager.postEvent(failEvent)
            pass
        finally:
            # update the execute menu items
            # i.e. disable execution for this pomset
            # now that execution has happened
            contextManager.app().GetTopWindow().updateExecuteMenus()
            

        return



    def OnCreateNewPomset(self, event):

        # TODO:
        # verify that there are no unsaved edits

        contextManager = self.getApp().contextManager()
        pomsetContext = contextManager.createNewPomset()

        guiEvent = PomsetCreatedEvent(
            object=pomsetContext,
            originalEvent=event,
            contextManager=contextManager)
        contextManager.postEvent(guiEvent)

        return


    def OnLoadExistingPomset(self, event):

        # TODO:
        # this code is the same as MenuEventHandler.OnLoadExistingPomset
        # should consolidate

        import pomsets_app.gui.utils as GuiUtilsModule
        try:
            window = self.eventSource()
            contextManager = self.getApp().contextManager()

            path = GuiUtilsModule.selectPathToLoadPomsetDefinition(
                contextManager, window, 
                title="Select a pomset to load" )

            if path is not None:
                pomsetContext = ContextModule.loadPomset(path=path)

                guiEvent = PomsetLoadedEvent(
                    object=pomsetContext,
                    originalEvent=event,
                    path=path,
                    contextManager=contextManager)

                contextManager.postEvent(guiEvent)

        except Exception, e:
            raise

        return


    def OnSaveCurrentPomset(self, event):

        # if the current pomset was loaded from file
        # then save back out to the same file

        # otherwise, treat this as a "save as"

        app = self.getApp()
        contextManager = app.contextManager()

        ContextModule.savePomset(contextManager.activePomsetContext())

        guiEvent = PomsetSavedEvent(
            object=contextManager.activePomsetContext(),
            contextManager = contextManager,
            originalEvent=self.event)
        contextManager.postEvent(guiEvent)

        return


    def OnSaveCurrentPomsetAs(self, event):

        import pomsets_app.gui.utils as GuiUtilsModule
        
        app = self.getApp()
        contextManager = app.contextManager()
        window = app.GetTopWindow()
        path = GuiUtilsModule.selectPathToSavePomsetDefinition(
            contextManager, window
            )

        if path is not None:
            ContextModule.savePomsetAs(
                contextManager.activePomsetContext(), 
                path)

            guiEvent = PomsetSavedEvent(
                object=contextManager.activePomsetContext(),
                contextManager = contextManager,
                originalEvent=self.event)
            contextManager.postEvent(guiEvent)

        return


    def OnCloseCurrentPomset(self, event):

        contextManager = self.getApp().contextManager()
        contextManager.closeActivePomset()

        return

    #def OnExitApplication(self, event):
    #    print "exiting application"
    #    return

    def OnAboutApplication(self, event):
        print "about application"
        import pomsets_app.gui.utils as GuiUtilsModule
        contextManager = self.getApp().contextManager()
        GuiUtilsModule.showAboutDialog(contextManager)
        return




    # END class MenuEventHandler
    pass


class MouseEventHandler(EventModule.MouseEventHandler):

    def OnLeftMouseButtonUpDefault(self, event):

        EventModule.MouseEventHandler.OnLeftMouseButtonUpDefault(self, event)

        contextManager = self.contextManager()
        frame = contextManager.app().GetTopWindow()

        # if the start of the drag
        # was from the library definitions (or loaded definitions)
        # then create a new node
        if hasattr(frame, 'dragSelection') and \
           frame.dragSelection is not None:

            frame.processDragSelection(event)

            pass


        return

    # END class MouseEventHandler
    pass




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'resources/qt/task output dialog.ui'
#
# Created: Sat Mar  6 10:52:07 2010
#      by: PyQt4 UI code generator 4.7
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(602, 486)
        self.buttonBox = QtGui.QDialogButtonBox(Dialog)
        self.buttonBox.setGeometry(QtCore.QRect(230, 440, 341, 32))
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtGui.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.splitter = QtGui.QSplitter(Dialog)
        self.splitter.setGeometry(QtCore.QRect(21, 20, 561, 411))
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setObjectName("splitter")
        self.treeTasks = QtGui.QTreeWidget(self.splitter)
        self.treeTasks.setMinimumSize(QtCore.QSize(0, 0))
        self.treeTasks.setEditTriggers(QtGui.QAbstractItemView.NoEditTriggers)
        self.treeTasks.setObjectName("treeTasks")
        self.treeTasks.headerItem().setText(0, "1")
        self.treeTasks.header().setVisible(True)
        self.textEditOutput = QtGui.QTextEdit(self.splitter)
        self.textEditOutput.setMinimumSize(QtCore.QSize(0, 0))
        self.textEditOutput.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOn)
        self.textEditOutput.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOn)
        self.textEditOutput.setLineWrapMode(QtGui.QTextEdit.WidgetWidth)
        self.textEditOutput.setReadOnly(True)
        self.textEditOutput.setObjectName("textEditOutput")

        self.retranslateUi(Dialog)
        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("accepted()"), Dialog.accept)
        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("rejected()"), Dialog.reject)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        Dialog.setWindowTitle(QtGui.QApplication.translate("Dialog", "Task message outputs", None, QtGui.QApplication.UnicodeUTF8))


if __name__ == "__main__":
    import sys
    app = QtGui.QApplication(sys.argv)
    Dialog = QtGui.QDialog()
    ui = Ui_Dialog()
    ui.setupUi(Dialog)
    Dialog.show()
    sys.exit(app.exec_())




from __future__ import with_statement

import logging
import os

import StringIO

from PyQt4.QtCore import *
from PyQt4 import QtGui

import pomsets_app.utils as AppUtilsModule

import pomsets_app.gui.qt as QtModule
import pomsets_app.gui.qt.application as ApplicationModule

class Controller(QtModule.Controller, QtGui.QDialog):

    LICENSE_FILES = ['gpl-2.0.txt',
                     'gpl-3.0.txt',
                     'commercial.txt']
    LICENSE_LABELS = ['Non-commercial | GPL v2',
                      'Non-commercial | GPL v3',
                      'Commercial | 30 day evaluation']
    
    def __init__(self, *args, **kwds):
        QtGui.QDialog.__init__(self, *args, **kwds)
        QtModule.Controller.__init__(self)

        return

    def populate(self):

        widget = self.widget()
        contextManager = self.contextManager()

        comboBoxLicense = widget.comboBoxLicense
        comboBoxLicense.addItems(Controller.LICENSE_LABELS)

        self.populateLicense()

        return

    def populateLicense(self):
        widget = self.widget()
        resourcePath = ApplicationModule.getDefaultResourcePath()

        licenseType = str(widget.comboBoxLicense.currentText())
        licenseIndex = Controller.LICENSE_LABELS.index(licenseType)
        licenseFile = Controller.LICENSE_FILES[licenseIndex]
        licensePath = os.path.join(resourcePath, 'licenses', licenseFile)
        
        textEditLicense = widget.textEditLicense
        with open(licensePath, 'r') as f:
            licenseText = ''.join(f.readlines())
            textEditLicense.setText(licenseText)
            pass

        return

    def userHasReadLicense(self):
        checkBoxRead = self.widget().checkBoxRead
        return checkBoxRead.isChecked()

    def saveUserAgreement(self):

        configDir = AppUtilsModule.getDefaultConfigDir()
        if not os.path.exists(configDir):
            os.makedirs(configDir)

        licenseFile = self.contextManager().app().getLicenseFile()
        if os.path.exists(licenseFile):
            return

        # this will create the file
        with open(licenseFile, 'w') as f:
            pass
        
        return


    @pyqtSlot()
    def on_buttonAgree_clicked(self):
        if not self.userHasReadLicense():
            # show a message box
            messageBox = QtGui.QMessageBox(self)
            messageBox.setText('Please check the box to indicate that you have read the license')
            messageBox.show()
            return

        self.saveUserAgreement()
        self.accept()
        return

    @pyqtSlot()
    def on_buttonDisagree_clicked(self):
        self.reject()
        return

    @pyqtSlot(str)
    def on_comboBoxLicense_currentIndexChanged(self, value):
        self.populateLicense()
        return



    # END class Controller
    pass



import functools

from PyQt4 import QtGui
from PyQt4.QtCore import *
from PyQt4.QtOpenGL import *

import logging
import os
import uuid

import numpy

import pypatterns.filter as FilterModule
import pypatterns.relational as RelationalModule

import pomsets.parameter as ParameterModule

import pomsets_app.utils as AppUtilsModule

import pomsets_app.gui.graph as GraphModule

class ContextualMenu(QtGui.QMenu):

    def position(self, value=None):
        if value is not None:
            self._position = value
        if not hasattr(self, '_position'):
            self._position = None
        return self._position

    def selection(self, value=None):
        if value is not None:
            self._selection = value
        if not hasattr(self, '_selection'):
            self._selection = []
        return self._selection

    def contextManager(self, value=None):
        if value is not None:
            self._contextManager = value
        if not hasattr(self, '_contextManager'):
            self._contextManager = None
        return self._contextManager


    # END class ContextualMenu
    pass

class CanvasContextualMenu(ContextualMenu):

    def bindEvents(self):
        contextManager = self.contextManager()
        pomsetContext, pomsetReferencePath, isEditable = \
            contextManager.currentDisplayedPomsetInfo()
        pomsetReference = pomsetReferencePath[-1]

        if isEditable and not contextManager.hasTaskForPomsetPath(pomsetReferencePath):
            self.addAction(QString('Create nest'), self.OnCreateNest)

            if pomsetContext is not None:
                self.addSeparator()
                self.addAction(QString('Auto arrange'), self.OnAutoArrange)

        return


    def OnAutoArrange(self):
        canvas = self.contextManager().mainWindow().canvas
        canvas.computeLayout()
        canvas.updateGL()
        return

    def OnCreateNest(self):
        contextManager = self.contextManager()
        pomsetContext, pomsetReferencePath, isEditable = \
            contextManager.currentDisplayedPomsetInfo()

        pomsetReference = pomsetReferencePath[-1]
        builder = contextManager.pomsetBuilder()

        # query the user for the name of the nest
        nodeName, status = QtGui.QInputDialog.getText(
            contextManager.mainWindow(),
            QString("Create nest:"),
            QString("Name:"))

        # create the node only if the user provided a name
        # and did not cancel the request
        if status:
            nodeName = str(nodeName)
            nestName = 'nest_%s' % uuid.uuid4().hex[:3]
            nestContext = builder.createNewNestPomset(name=nestName)
            nestDefinition = nestContext.pomset()

            contextManager.mainWindow().OnCreateNode(
                pomsetContext, pomsetReference,
                nestDefinition, self.position(),
                name=nodeName)

        return

    # END class CanvasContextualMenu
    pass


class CanvasContextualMenuWithCreateNodeEnabled(CanvasContextualMenu):

    def bindEvents(self):
        contextManager = self.contextManager()
        pomsetContext, pomsetReferencePath, isEditable = \
            contextManager.currentDisplayedPomsetInfo()

        if isEditable:
            pomsetReference = pomsetReferencePath[-1]
            if pomsetContext is not None and \
                    not contextManager.hasTaskForPomsetPath(pomsetReferencePath):
                self.addAction(QString('Create node'), self.OnCreateNode)
                self.addSeparator()

            CanvasContextualMenu.bindEvents(self)

        return

    def OnCreateNode(self):
        contextManager = self.contextManager()
        pomsetContext, pomsetReferencePath, isEditable = \
            contextManager.currentDisplayedPomsetInfo()
        pomsetReference = pomsetReferencePath[-1]

        # import the modules
        import pomsets_app.gui.qt.select_definition.widget as WidgetModule
        import pomsets_app.gui.qt.select_definition.controller as ControllerModule

        # initialize the widget
        controller = ControllerModule.Controller(self)
        controller.contextManager(contextManager)
        controller.pomsetContext(pomsetContext)
        controller.pomsetReference(pomsetReference)
        controller.canvasPosition(self.position())

        ui = WidgetModule.Ui_Dialog()
        ui.setupUi(controller)

        controller.widget(ui)

        # populate the dialog
        controller.populate()

        controller.show()

        return

    # END class CanvasContextualMenuWithCreateNodeEnabled
    pass


class MultipleSelectionContextualMenu(ContextualMenu):

    def bindEvents(self):
        self.addAction(QString('Duplicate'), self.OnDuplicate)
        self.addAction(QString('Delete'), self.OnDelete)
        return

    def OnDuplicate(self):
        contextManager = self.contextManager()
        contextManager.mainWindow().OnDuplicateSelection(self.selection())
        return


    def OnDelete(self):

        contextManager = self.contextManager()
        builder = contextManager.pomsetBuilder()

        contextManager.mainWindow().OnDeleteSelection(self.selection())
        return

    # END class MultipleSelectionContextualMenu
    pass


class NodeContextualMenu(ContextualMenu):

    def bindEvents(self):

        contextManager = self.contextManager()

        pomsetContext, pomsetReferencePath, isEditable = \
            contextManager.currentDisplayedPomsetInfo()
        if not contextManager.hasTaskForPomsetPath(pomsetReferencePath):
 
            if isEditable:
                self.addAction(QString('Edit task'), self.OnEditTask)

                # should also add "Edit definition"
                # if the definition is not a library definition
                dataNode = self.selection()[0].nodeData
                if not dataNode.definitionToReference().isLibraryDefinition():
                    self.addAction(QString('Edit definition'), 
                                   self.OnEditTaskDefinition)
                else:
                    self.addAction(QString('View definition'), 
                                   self.OnViewTaskDefinition)

                self.addAction(QString('Duplicate'), self.OnDuplicate)
                self.addAction(QString('Delete'), self.OnDelete)
            else:
                self.addAction(QString('View definition'), 
                               self.OnViewTaskDefinition)
        else:
            # enabled only if the node has executed or errored out
            # and there's only one node selected
            if len(self.selection()) == 1:
                self.addAction(QString('View output messages'), 
                               self.OnViewOutputMessages)

        return


    def OnEditTask(self):

        contextManager = self.contextManager()

        guiNode = self.selection()[0]
        definitionToEdit = guiNode.nodeData

        pomsetContext, pomsetReferencePath, isEditable = \
            contextManager.currentDisplayedPomsetInfo()

        self.contextManager().mainWindow().OnEditTask(
            pomsetContext, definitionToEdit)

        return

    
    def OnViewTaskDefinition(self):
        self.OnDisplayTaskDefinition(False)
        return


    def OnEditTaskDefinition(self):
        self.OnDisplayTaskDefinition(True)
        return


    def OnDisplayTaskDefinition(self, allowEdit):

        contextManager = self.contextManager()

        guiNode = self.selection()[0]
        dataNode = guiNode.nodeData
        definitionToEdit = dataNode.definitionToReference()

        # now determine if the definition is in the transient library
        # or defined internally in the pomset
        # and then get the pomset context associated with that definition
        filter = RelationalModule.ColumnValueFilter(
            'definition',
            FilterModule.IdentityFilter(definitionToEdit))
        library = contextManager.transientLibrary()

        pomsetContext, pomsetReferencePath, isEditable = \
            contextManager.currentDisplayedPomsetInfo()

        if library.hasDefinition(filter):
            contexts = RelationalModule.Table.reduceRetrieve(
                library.definitionTable(), filter, ['context'], [])
            pomsetContext = contexts[0]
            pomsetReferencePath = [pomsetContext.reference()]
            pass
        pomsetReferencePath = pomsetReferencePath + [dataNode]

        print "calling frame.OnDisplayDefinition"
        allowEdit = allowEdit and isEditable
        self.contextManager().mainWindow().OnDisplayDefinition(
            pomsetContext, pomsetReferencePath, allowEdit=allowEdit)
        return


    def OnDelete(self):
        contextManager = self.contextManager()
        contextManager.mainWindow().OnDeleteNodes(self.selection())
        return

    def OnDuplicate(self):
        contextManager = self.contextManager()
        contextManager.mainWindow().OnDuplicateNodes(self.selection())
        return

    def OnViewOutputMessages(self):

        contextManager = self.contextManager()
        pomsetContext, pomsetReferencePath, isEditable = \
            contextManager.currentDisplayedPomsetInfo()

        # import the modules
        import pomsets_app.gui.qt.task_output.widget as WidgetModule
        import pomsets_app.gui.qt.task_output.controller as ControllerModule

        # initialize the widget
        controller = ControllerModule.Controller(self)
        controller.contextManager(contextManager)
        controller.pomsetContext(pomsetContext)

        ui = WidgetModule.Ui_Dialog()
        ui.setupUi(controller)

        controller.widget(ui)

        # populate the dialog
        controller.populate()

        controller.show()


        return

    # END class NodeContextualMenu
    pass


class EdgeContextualMenu(ContextualMenu):

    def bindEvents(self):
        self.addAction(QString('Disconnect edge'), self.OnDisconnect)
        return

    def OnDisconnect(self):

        contextManager = self.contextManager()
        builder = contextManager.pomsetBuilder()

        canvas = self.contextManager().mainWindow().canvas
        clickables = canvas.getClickableObjectsAtCanvasCoordinates(
            self.position().x(), self.position().y())
        edge = None
        for clickable in clickables:
            if not isinstance(clickable, GraphModule.Edge):
                continue
            edge = clickable
            break
        if edge is None:
            # TODO:
            # should display error box to user
            return

        contextManager.mainWindow().OnDeleteEdges([edge])
        return

    # END class EdgeContextualMenu
    pass


class PortContextualMenu(ContextualMenu):

    def getDataNode(self):
        selected = self.selection()[0]
        dataNode = selected.node.nodeData
        return dataNode

    def getParameter(self):
        dataNode = self.getDataNode()

        selected = self.selection()[0]
        parameterId = selected.name
        parameter = dataNode.getParameter(parameterId)
        return parameter

    def parameterIsData(self):
        parameter = self.getParameter()

        return parameter.portType() == ParameterModule.PORT_TYPE_DATA


    def bindEvents(self):
        contextManager = self.contextManager()
        pomsetContext, pomsetReferencePath, isEditable = \
            contextManager.currentDisplayedPomsetInfo()
        if not isEditable:
            return

        if not self.parameterIsData():
            return

        if isEditable and not contextManager.hasTaskForPomsetPath(pomsetReferencePath):
            self.addAction(QString('Edit value'), self.OnEditValue)

            parameter = self.getParameter()

            # TODO:
            # determine if the parameter has already been exposed
            # if so, there no need to do so
            dataNode = self.getDataNode()
            containingNest = dataNode.graph()
            if containingNest.exposesNodeParameter(dataNode, parameter.id()):
                # add a "Unexpose parameter" menu item instead
                pass
            else:
                self.addSeparator()
                self.addAction(QString('Expose to new parameter on nest'), 
                               self.OnExposeAsNewParameter)

                # if a parameter is a side effect, i.e. output file,
                # or just an output
                # then we can can only expose as a new parameter
                # so don't do anything here
                # otherwise, determine if there are existing exposed parameters
                # and add them as possible choices
                if not (parameter.getAttribute(ParameterModule.PORT_ATTRIBUTE_ISSIDEEFFECT) or 
                        parameter.portDirection()==ParameterModule.PORT_DIRECTION_OUTPUT):

                    filter = FilterModule.constructAndFilter()
                    filter.addFilter(
                        FilterModule.ObjectKeyMatchesFilter(
                            filter=FilterModule.EquivalenceFilter(ParameterModule.PORT_DIRECTION_INPUT),
                            keyFunction=lambda x: x.portDirection()
                            )
                        )

                    notTemporalFilter = FilterModule.constructNotFilter()
                    notTemporalFilter.addFilter(
                        FilterModule.EquivalenceFilter(ParameterModule.PORT_TYPE_TEMPORAL)
                        )
                    filter.addFilter(
                        FilterModule.ObjectKeyMatchesFilter(
                            keyFunction = lambda x: x.portType(),
                            filter = notTemporalFilter
                            )
                        )

                    nest = pomsetReferencePath[-1].definitionToReference()
                    nestParameters = [x for x in nest.getParametersByFilter(filter)]
                    if len(nestParameters) is not 0:
                        submenu = self.addMenu(QString('Connect to existing nest parameter'))
                        for nestParameter in nestParameters:
                            nestParameterName = nestParameter.id()
                            function = functools.partial(self.OnConnectToExistingParameter,
                                                         parameterName=nestParameterName)
                            submenu.addAction(QString(nestParameterName),
                                              function)
                        pass

                    pass
                pass
            pass

        return


    def OnExposeAsNewParameter(self):
        contextManager = self.contextManager()
        pomsetContext, pomsetReferencePath, isEditable = \
            contextManager.currentDisplayedPomsetInfo()
        pomsetReference = pomsetReferencePath[-1]

        dataNode = self.getDataNode()
        selected = self.selection()[0]
        parameterId = selected.name

        # TODO:
        # query the user for name of the exposed parameter
        # query the user for the name of the nest
        exposedParameterId, status = QtGui.QInputDialog.getText(
            contextManager.mainWindow(),
            QString("Expose node \"%s\"'s parameter \"%s\" to nest" % 
                    (dataNode.name(), parameterId)),
            QString("Exposed name:"))

        if status:
            exposedParameterId = str(exposedParameterId)

            logging.debug("exposing node \"%s\"'s parameter \"%s\" to nest as \"%s\"" % (dataNode.name(), parameterId, exposedParameterId))

            contextManager.pomsetBuilder().exposeNodeParameter(
                pomsetReference.definitionToReference(),
                exposedParameterId,
                dataNode, parameterId,
                shouldCreate=True
                )

        return


    def OnConnectToExistingParameter(self, parameterName=None):

        exposedParameterId = parameterName

        contextManager = self.contextManager()
        pomsetContext, pomsetReferencePath, isEditable = \
            contextManager.currentDisplayedPomsetInfo()
        pomsetReference = pomsetReferencePath[-1]

        dataNode = self.getDataNode()
        selected = self.selection()[0]
        parameterId = selected.name

        # TODO:
        # find the exposed parameters that fits
        # the attributes of the selected parameter
        contextManager.pomsetBuilder().exposeNodeParameter(
            pomsetReference.definitionToReference(),
            exposedParameterId,
            dataNode, parameterId,
            shouldCreate=False
            )

        return
    

    def OnEditValue(self):

        contextManager = self.contextManager()
        pomsetContext, pomsetReferencePath, isEditable = \
            contextManager.currentDisplayedPomsetInfo()

        dataNode = self.getDataNode()

        parameter = self.getParameter()
        contextManager.mainWindow().OnEditParameterBinding(
            pomsetContext, dataNode, parameter)

        return

    # END class PortContextualMenu
    pass


class ActivePomsetContextualMenu(ContextualMenu):

    def bindEvents(self):
        self.addAction(QString('New'), self.OnCreate)

        self.addAction(QString('Load'), self.OnLoad)

        pomsetContext, pomset = self.selection()
        if pomsetContext is not None:

            self.addAction(QString("Rename"), self.OnRename)

            # need a "save" and "save as"
            if pomsetContext.url() is not None:
                self.addAction(QString('Save'), self.OnSave)

            self.addAction(QString('Save as'), self.OnSaveAs)

            self.addAction(QString('Close'), self.OnClose)

            pass


        return

    def OnRename(self):

        # TODO:
        # get the name from the user
        newName, status = QtGui.QInputDialog.getText(
            self,
            QString("Rename pomset"),
            QString("New name:"))

        if status:
            newName = str(newName)
            if len(newName):
                self.contextManager().mainWindow().OnRenameTopLevelPomset(newName)
        return

    def OnCreate(self):
        self.contextManager().mainWindow().OnCreateNewTopLevelPomset()
        return

    def OnLoad(self):
        self.contextManager().mainWindow().OnLoadTopLevelPomset()
        return

    def OnSave(self):
        context = self.selection()[0]
        self.contextManager().mainWindow().OnSavePomset(
            context, context.url())
        return

    def OnSaveAs(self):
        context = self.selection()[0]
        self.contextManager().mainWindow().OnSavePomset(context)
        return

    def OnClose(self):
        contextManager = self.contextManager()
        contextManager.mainWindow().OnCloseActivePomset(self.selection()[0])

        return

    # END class ActivePomsetContextualMenu
    pass


class LibraryDefinitionContextualMenu(ContextualMenu):
    
    def bindEvents(self):

        pomsetContext, pomset = self.selection()
        if pomsetContext is not None:
            self.addAction('View', self.OnViewDefinition)
            # add the "Delete menu item
            self.addAction('Remove', self.OnRemoveDefinition)
            
        return

    def OnViewDefinition(self):
        pomsetContext, pomsetReference = self.selection()
        contextManager = self.contextManager()
        contextManager.mainWindow().OnDisplayDefinition(
            pomsetContext, pomsetReference, allowEdit=False)
        return


    def OnRemoveDefinition(self):
        """
        TODO: move this out of the menu
        """

        pomsetContext, pomsetReference = self.selection()
        pomset = pomsetReference.definitionToReference()
        contextManager = self.contextManager()


        persistentLibrary = contextManager.persistentLibrary()
        
        # backup the existing library
        AppUtilsModule.backupLibrary(
            persistentLibrary.bootstrapLoaderDefinitionsDir())
        
        definitionFilter = RelationalModule.ColumnValueFilter(
            'definition',
            FilterModule.IdentityFilter(pomset)
        )
        
        # remove from transient library
        persistentLibrary.removeDefinition(definitionFilter)
        
        # remove the definition from the file system
        definitionUrl = os.path.sep.join([
                persistentLibrary.bootstrapLoaderDefinitionsDir(),
                pomset.url()])

        if not os.path.exists(definitionUrl):
            logging.error('cannot remove definition from url %s' % definitionUrl)
            return
        else:
            os.unlink(definitionUrl)
            
        # save out the new bootstrapper pomset
        # that will not load the just removed pomset
        persistentLibrary.saveBootstrapLoaderPomset()

        contextManager.mainWindow().OnDefinitionRemoved(
            persistentLibrary, pomsetContext)

        return


    # END class LibraryDefinitionContextualMenu
    pass



class LoadedDefinitionContextualMenu(ContextualMenu):


    def bindEvents(self):
        self.addAction(QString('New'), self.OnCreateAtomicDefinition)
        # self.addAction(QString('New nest'), self.OnCreateNestDefinition)

        pomsetContext, pomset = self.selection()

        if pomsetContext is not None:
            # add the "Edit" menu item
            # add the "Delete menu item
            self.addAction(QString('Edit'), self.OnEditDefinition)
            self.addAction(QString('Remove'), self.OnRemoveDefinition)

            self.addSeparator()
            self.addAction(QString('Add to library'), self.OnAddDefinitionToLibrary)
            
            # TODO:
            # need a "save" and "save as"
            self.addAction(QString('Save'), self.OnSave)
            self.addAction(QString('Save as'), self.OnSaveAs)
            pass

        self.addSeparator()
        self.addAction(QString('Load from file'), self.OnLoadFromFile)

        return

    def OnCreateAtomicDefinition(self):
        self.contextManager().mainWindow().OnCreateNewAtomicPomset()
        return

    def OnCreateNestDefinition(self):
        # self.contextManager().mainWindow().OnCreateNewNestPomset()
        print "should create new nest pomset"
        return

    def OnEditDefinition(self):
        pomsetContext, pomsetReference = self.selection()
        self.contextManager().mainWindow().OnDisplayDefinition(
            pomsetContext, pomsetReference, allowEdit=True)
        return


    def OnRemoveDefinition(self):

        pomsetContext, pomsetReference = self.selection()
        pomset = pomsetReference.definitionToReference()

        contextManager = self.contextManager()
        transientLibrary = contextManager.transientLibrary()


        # remove from transient library
        definitionFilter = RelationalModule.ColumnValueFilter(
            'definition',
            FilterModule.IdentityFilter(pomset)
        )
        transientLibrary.removeDefinition(definitionFilter)

        contextManager.mainWindow().OnDefinitionRemoved(
            transientLibrary, pomsetContext)
        return


    def OnAddDefinitionToLibrary(self):
        context = self.selection()[0]
        self.contextManager().mainWindow().OnMoveDefinitionToLibrary(context)
        return

    def OnSave(self):
        context = self.selection()[0]
        self.contextManager().mainWindow().OnSavePomset(
            context, context.url())
        return

    def OnSaveAs(self):
        context = self.selection()[0]
        self.contextManager().mainWindow().OnSavePomset(context)
        return

    def OnLoadFromFile(self):
        contextManager = self.contextManager()
        transientLibrary = contextManager.transientLibrary()

        filter = FilterModule.ObjectKeyMatchesFilter(
            filter = FilterModule.IdentityFilter(True),
            keyFunction = lambda x: x.pomset().isAtomic()
            )
        contextManager.mainWindow().OnLoadPomsetIntoLibrary(
            transientLibrary, filter=filter)
        return

    # END class LoadedDefinitionContextualMenu
    pass



import logging

from PyQt4.QtCore import *
from PyQt4 import QtGui

import pypatterns.filter as FilterModule
import pypatterns.relational as RelationalModule

import pomsets.parameter as ParameterModule

import pomsets_app.gui.qt as QtModule

class Controller(QtModule.Controller, QtGui.QDialog):

    def definition(self, value=None):
        if value is not None:
            self._definition = value
        if not hasattr(self, '_definition'):
            self._definition = None
        return self._definition

    def parameter(self, value=None):
        if value is not None:
            self._parameter = value
        if not hasattr(self, '_parameter'):
            self._parameter = None
        return self._parameter


    def __init__(self, *args, **kwds):
        QtGui.QDialog.__init__(self, *args, **kwds)
        QtModule.Controller.__init__(self)
        return


    def populate(self):

        widget = self.widget()
        contextManager = self.contextManager()

        definition = self.definition()
        parameter = self.parameter()

        self.setWindowTitle('Specify parameter sweep group for "%s"' % parameter.id())

        buttonToSelect = widget.buttonUseOwnGroup
        if not definition.parameterIsInOwnParameterSweepGroup(parameter.id()):
            buttonToSelect = widget.buttonUseOtherGroup

        buttonToSelect.setChecked(True)


        listWidget = widget.listWidget

        # first get all the parameter sweep groups
        parameterSweepGroups = list(set(
                RelationalModule.Table.reduceRetrieve(
                    definition.parameterSweepGroups(),
                    FilterModule.TRUE_FILTER, ['group'], [])))
        psgMap = {}
        for psg in parameterSweepGroups:
            for parameterId in psg:
                psgMap[parameterId] = psg

        # now get the parameters that are in their own parameter sweep groups
        parameterFilter = FilterModule.constructAndFilter()
        parameterFilter.addFilter(
            FilterModule.ObjectKeyMatchesFilter(
                filter=FilterModule.EquivalenceFilter(ParameterModule.PORT_TYPE_DATA),
                keyFunction = lambda x: x.portType()
                )
            )
        notThisParameterFilter = FilterModule.constructNotFilter()
        notThisParameterFilter.addFilter(
            FilterModule.IdentityFilter(parameter))
        parameterFilter.addFilter(notThisParameterFilter)

        allParameters = definition.getParametersByFilter(parameterFilter)
        for allParameter in allParameters:
            parameterId = allParameter.id()
            if not parameterId in psgMap:
                group = tuple([parameterId])
                psgMap[parameterId] = group
                parameterSweepGroups.append(group)

        for psg in parameterSweepGroups:
            item = QtGui.QListWidgetItem('group')
            item.setData(Qt.UserRole, psg)
            item.setData(Qt.DisplayRole, str(psg))
            listWidget.addItem(item)
            pass

        return




    @pyqtSlot(QtGui.QAbstractButton)
    def on_buttonBox_clicked(self, button):
        widget = self.widget()
        buttonRole = widget.buttonBox.buttonRole(button)
        if buttonRole == QtGui.QDialogButtonBox.ApplyRole:
            self.saveValues()
            pass
        if buttonRole == QtGui.QDialogButtonBox.AcceptRole:
            self.saveValues()
            self.accept()
            pass
        if buttonRole == QtGui.QDialogButtonBox.RejectRole:
            self.reject()
            pass
        return


    def saveValues(self):

        widget = self.widget()
        contextManager = self.contextManager()
        pomsetContext = self.pomsetContext()
        definition = self.definition()
        parameter = self.parameter()
        parameterId = parameter.id()

        buttonUseOwnGroup = widget.buttonUseOwnGroup

        needToRemoveFromGroup = False
        if not definition.parameterIsInOwnParameterSweepGroup(parameterId):
            if buttonUseOwnGroup.isChecked():
                needToRemoveFromGroup = True
            else:
                if not parameterId in selectedGroup:
                    needToRemoveFromGroup = True
                pass
            pass

        buttonUseOtherGroup = widget.buttonUseOtherGroup
        needToAddToGroup = False
        if buttonUseOtherGroup.isChecked():
            listWidget = widget.listWidget
            selectedItems = list(listWidget.selectedItems())
            if len(selectedItems) == 0:
                return
            selectedItem = selectedItems[0]
            selectedGroup = tuple(
                [str(x) 
                 for x in selectedItem.data(Qt.UserRole).toPyObject()])

            if definition.parameterIsInOwnParameterSweepGroup(parameterId):
                needToAddToGroup = True
            else:
                if not parameterId in selectedGroup:
                    needToAddToGroup = True
            pass
                    

        if needToRemoveFromGroup:
            group = definition.getGroupForParameterSweep(parameterId)
            definition.removeParameterSweepGroup(group)
            newGroup = [x for x in list(group) if not x==parameterId]
            definition.addParameterSweepGroup(newGroup)

        if needToAddToGroup:
            definition.removeParameterSweepGroup(selectedGroup)
            newGroup = list(selectedGroup) + [parameterId]
            definition.addParameterSweepGroup(newGroup)
            

        if needToRemoveFromGroup or needToAddToGroup:
            pomsetContext.isModified(True)
            contextManager.mainWindow().emit(
                SIGNAL("OnPomsetModified(PyQt_PyObject)"), pomsetContext)
            pass

        return


    # END class Controller
    pass



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'resources/qt/select definition dialog.ui'
#
# Created: Thu Mar  4 16:19:49 2010
#      by: PyQt4 UI code generator 4.7
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(381, 436)
        self.buttonBox = QtGui.QDialogButtonBox(Dialog)
        self.buttonBox.setGeometry(QtCore.QRect(20, 390, 341, 32))
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtGui.QDialogButtonBox.Cancel|QtGui.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.labelName = QtGui.QLabel(Dialog)
        self.labelName.setGeometry(QtCore.QRect(30, 350, 62, 16))
        self.labelName.setObjectName("labelName")
        self.treeWidgetDefinitions = QtGui.QTreeWidget(Dialog)
        self.treeWidgetDefinitions.setGeometry(QtCore.QRect(20, 20, 341, 301))
        self.treeWidgetDefinitions.setObjectName("treeWidgetDefinitions")
        self.treeWidgetDefinitions.headerItem().setText(0, "1")
        self.treeWidgetDefinitions.header().setVisible(False)
        self.textEditName = QtGui.QLineEdit(Dialog)
        self.textEditName.setGeometry(QtCore.QRect(80, 341, 281, 31))
        self.textEditName.setObjectName("textEditName")

        self.retranslateUi(Dialog)
        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("accepted()"), Dialog.accept)
        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("rejected()"), Dialog.reject)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        Dialog.setWindowTitle(QtGui.QApplication.translate("Dialog", "Dialog", None, QtGui.QApplication.UnicodeUTF8))
        self.labelName.setText(QtGui.QApplication.translate("Dialog", "Name", None, QtGui.QApplication.UnicodeUTF8))


if __name__ == "__main__":
    import sys
    app = QtGui.QApplication(sys.argv)
    Dialog = QtGui.QDialog()
    ui = Ui_Dialog()
    ui.setupUi(Dialog)
    Dialog.show()
    sys.exit(app.exec_())




from PyQt4.QtCore import *

class Controller(object):

    def contextManager(self, value=None):
        if value is not None:
            self._contextManager = value
        if not hasattr(self, '_contextManager'):
            self._contextManager = None
        return self._contextManager

    def pomsetContext(self, value=None):
        if value is not None:
            self._pomsetContext = value
        if not hasattr(self, '_pomsetContext'):
            self._pomsetContext = None
        return self._pomsetContext

    def pomsetReference(self, value=None):
        if value is not None:
            self._pomsetReference = value
        if not hasattr(self, '_pomsetReference'):
            self._pomsetReference = None
        return self._pomsetReference

    def widget(self, value=None):
        if value is not None:
            self._widget = value
        if not hasattr(self, '_widget'):
            self._widget = None
        return self._widget

    def connectSignals(self):
        raise NotImplementedError('%s.connectSignals' % self.__class__)

    # END class Controller
    pass



import logging
import os

from PyQt4.QtCore import *
from PyQt4 import QtGui

import pypatterns.filter as FilterModule

import pomsets.parameter as ParameterModule
import pomsets.task as TaskModule

import pomsets_app.utils as AppUtilsModule
import pomsets_app.gui.qt as QtModule

class Controller(QtModule.Controller, QtGui.QDialog):


    def __init__(self, *args, **kwds):
        QtGui.QDialog.__init__(self, *args, **kwds)
        QtModule.Controller.__init__(self)

        return

    def getDefaultText(self):
        value = "Click on individual child tasks to see their output"
        return value

    def populate(self):

        widget = self.widget()
        contextManager = self.contextManager()

        splitter = widget.splitter

        treeTasks = widget.treeTasks
        
        pomsetContext = self.pomsetContext()
        pomset = pomsetContext.pomset()
        pomsetReference = pomsetContext.reference()
        task = contextManager.getTaskForPomsetPath([pomsetReference])

        # populate the tree
        treeTasks.setHeaderLabels(['Tasks'])
        item = QtGui.QTreeWidgetItem([pomset.name()])
        item.setData(0, Qt.UserRole, task)
        item.setData(0, Qt.DisplayRole, pomset.name())
        treeTasks.addTopLevelItem(item)
        self.addChildTasks(task, item)
        
        treeTasks.expandAll()

        handle = splitter.handle(0)

        textEditOutput = widget.textEditOutput
        value = self.getDefaultText()
        textEditOutput.setText(value)

        return


    def addChildTasks(self, task, parentItem):
        widget = self.widget()
        contextManager = self.contextManager()
        treeTasks = widget.treeTasks

        
        for index, childTask in enumerate(task.getChildTasks()):
            name = childTask.definition().name()
            item = QtGui.QTreeWidgetItem([name])
            item.setData(0, Qt.UserRole, childTask)

            if task.isParameterSweepTasksHolder():
                name = 'task %s' % index
            item.setData(0, Qt.DisplayRole, name)

            parentItem.addChild(item)
            if isinstance(childTask, TaskModule.CompositeTask):
                self.addChildTasks(childTask, item)
            pass

        return


    @pyqtSlot(QtGui.QTreeWidgetItem, int)
    def on_treeTasks_itemClicked(self, item, int):

        task = item.data(0, Qt.UserRole).toPyObject()

        if isinstance(task, TaskModule.CompositeTask):
            value = self.getDefaultText()
            pass
        else:
            request = task.workRequest()
            stream = request.kwds.get('process message output', None)

            lines = []

            # TODO:
            # print worker thread information

            executedCommand = request.kwds.get('executed command', None)
            if executedCommand:
                lines.append('# executed command\n')
                lines.append('# %s \n' % ' '.join(executedCommand))
                lines.append('\n')

            outputFound = False
            if stream is not None:
                streamLines = stream.readlines()
                if len(streamLines):
                    lines.extend(streamLines)
                    outputFound = True
                pass

            
            if not outputFound:
                lines.append("# No output messages found\n")

            if request.kwds.get('exception stack trace', None) is not None:
                exceptionStackTrace = request.kwds.get('exception stack trace')
                lines.extend(exceptionStackTrace)

            value = ''.join(lines)
            pass

        widget = self.widget()
        textEditOutput = widget.textEditOutput
        textEditOutput.setText(value)


        return




    # END class Controller
    pass



import logging

from PyQt4.QtCore import *
from PyQt4 import QtGui

import pypatterns.filter as FilterModule

import pomsets.parameter as ParameterModule

import pomsets_app.gui.qt as QtModule

class Controller(QtModule.Controller, QtGui.QDialog):

    def definition(self, value=None):
        if value is not None:
            self._definition = value
        if not hasattr(self, '_definition'):
            self._definition = None
        return self._definition

    def parameter(self, value=None):
        if value is not None:
            self._parameter = value
        if not hasattr(self, '_parameter'):
            self._parameter = None
        return self._parameter


    def __init__(self, *args, **kwds):
        QtGui.QDialog.__init__(self, *args, **kwds)
        QtModule.Controller.__init__(self)
        return

    def populate(self):

        widget = self.widget()
        contextManager = self.contextManager()

        definition = self.definition()
        parameter = self.parameter()

        self.setWindowTitle('Edit bindings for parameter "%s"' % parameter.id())
        
        checkBoxSweep = widget.checkBoxSweep
        isSweep = definition.isParameterSweep(parameter.id())
        checkBoxSweep.setChecked(isSweep)

        # populate the values 
        (dataNode, parameterToEdit) = \
            self.pomsetContext().getParameterToEdit(definition, parameter.id())

        values = ''
        if dataNode.hasParameterBinding(parameterToEdit.id()):
            values = '\n'.join(dataNode.getParameterBinding(parameterToEdit.id()))
        textEditValues = widget.textEditValues
        textEditValues.setPlainText(values)

        return




    @pyqtSlot(QtGui.QAbstractButton)
    def on_buttonBox_clicked(self, button):
        widget = self.widget()
        buttonRole = widget.buttonBox.buttonRole(button)
        if buttonRole == QtGui.QDialogButtonBox.ApplyRole:
            self.saveValues()
            pass
        if buttonRole == QtGui.QDialogButtonBox.AcceptRole:
            self.saveValues()
            self.accept()
            pass
        if buttonRole == QtGui.QDialogButtonBox.RejectRole:
            self.reject()
            pass
        return


    def saveValues(self):

        widget = self.widget()
        contextManager = self.contextManager()
        pomsetContext = self.pomsetContext()
        definition = self.definition()
        parameter = self.parameter()

        hasBeenModified = False

        checkBoxSweep = widget.checkBoxSweep
        isSweep = checkBoxSweep.isChecked()
        if not definition.isParameterSweep(parameter.id()) == isSweep:
            definition.isParameterSweep(parameter.id(), isSweep)
            hasBeenModified = True

        (dataNode, parameterToEdit) = \
            self.pomsetContext().getParameterToEdit(definition, parameter.id())


        textEditValues = widget.textEditValues
        values = str(textEditValues.toPlainText()).split('\n')
        if not dataNode.hasParameterBinding(parameterToEdit.id()) or \
                not values == dataNode.getParameterBinding(parameterToEdit.id()):
            dataNode.setParameterBinding(parameterToEdit.id(),
                                         values)
            hasBeenModified = True

        if hasBeenModified:
            pomsetContext.isModified(True)
            contextManager.mainWindow().emit(
                SIGNAL("OnPomsetModified(PyQt_PyObject)"), pomsetContext)
            pass

        return


    # END class Controller
    pass



import logging
import uuid
import wx

import pypatterns.filter as FilterModule
import pypatterns.relational as RelationalModule

import pomsets.library as LibraryModule

import pomsets_app.gui.event as EventModule
import pomsets_app.gui.widget as WidgetModule

import pomsets.parameter as ParameterModule


ID_DIALOG_SELECTDEFINITION = wx.NewId()

class SelectDefinitionDialog(WidgetModule.Dialog):
    
    XRC_NAME = 'SelectDefinitionDialog'

    def __init__(self, xmlResource, parent=None):
        WidgetModule.Dialog.__init__(
            self, 
            xmlResource.LoadDialog(parent, SelectDefinitionDialog.XRC_NAME))
        self.Fit()
        self.SetAutoLayout(True)

        treeCtrl = self.getWidget('definitionTreeCtrl')
        treeCtrl.SetSize(wx.Size(275, 400))

        self.SetSize(wx.Size(300, 450))

        return

    def populate(self):
        contextManager = self.contextManager

        treeCtrl = self.getWidget('definitionTreeCtrl')
        root = treeCtrl.AddRoot('Definitions')


        libraryRoot = treeCtrl.AppendItem(
            root, 'Library definitions')
        for definition in contextManager.getLibraryDefinitions():
            if definition.id() in [LibraryModule.ID_LOADLIBRARYDEFINITION,
                                   LibraryModule.ID_BOOTSTRAPLOADER]:
                continue
            name = definition.name() or "Anonymous"
            treeCtrl.AppendItem(
                libraryRoot, name, data=wx.TreeItemData(definition))
            pass

        loadedRoot = treeCtrl.AppendItem(
            root, 'Loaded definitions')
        for definition in contextManager.getLoadedDefinitions():
            name = definition.name() or "Anonymous"
            treeCtrl.AppendItem(
                loadedRoot, name, data=wx.TreeItemData(definition))
            pass

        treeCtrl.ExpandAll()

        return
    
    def OnButtonCancel(self, event):

        # close the dialog box with return code cancel
        self.EndModal(wx.ID_CANCEL)
        return

    def OnButtonOk(self, event):

        nameTextCtrl = self.getWidget('nameTextCtrl')
        name = nameTextCtrl.GetValue()

        treeCtrl = self.getWidget('definitionTreeCtrl')
        selectedItem = treeCtrl.GetSelection()
        selectedDefinition = None
        if selectedItem.IsOk():
            data = treeCtrl.GetItemData(selectedItem)
            selectedDefinition = data.GetData()
            selectedDefinition = selectedDefinition

        if selectedDefinition is None:
            self.EndModal(wx.ID_CANCEL)
            return

        self.nodeName = name
        self.selectedDefinition = selectedDefinition

        self.EndModal(wx.ID_OK)
        return

    # END SelectDefinitionDialog
    pass




class EditAtomicDefinitionDialog(WidgetModule.Dialog):
    XRC_NAME = 'EditDefinitionDialog'
    WIDGET_BINDINGS = [
        ('wxID_OK', 'OnButtonOk', wx.EVT_BUTTON),
        ('wxID_APPLY', 'OnButtonApply', wx.EVT_BUTTON),
        ('buttonAdd', 'OnAddParameter', wx.EVT_BUTTON),
        ('buttonEdit', 'OnEditParameter', wx.EVT_BUTTON),
        ('buttonRemove', 'OnRemoveParameter', wx.EVT_BUTTON),
        ('choiceExecutableType', 'OnChoiceExecutable', wx.EVT_CHOICE)
    ]


    def __init__(self, xmlResource, parent=None):
        WidgetModule.Dialog.__init__(
            self, 
            xmlResource.LoadDialog(parent, EditAtomicDefinitionDialog.XRC_NAME))
        self.Fit()
        self.SetAutoLayout(True)

        self.listBoxParameters = self.getWidget('listBoxParameters')
        self.definitionToEdit = None

        self.SetSize(wx.Size(300, 450))

        return

    def populate(self):

        # first populate the possible executable types
        choiceExecutableType = self.getWidget('choiceExecutableType')

        contextManager = self.contextManager
        executableTypes = contextManager.getExecutableTypes()
        map(choiceExecutableType.Append, executableTypes)
        self.executableTypes = executableTypes
        return


    def setObjectToEdit(self, contextToEdit):

        self.contextToEdit = contextToEdit

        definitionToEdit = contextToEdit.pomset()
        assert definitionToEdit is not None
        self.definitionToEdit = definitionToEdit

        textCtrlName = self.getWidget('textCtrlName')
        textCtrlName.SetValue(definitionToEdit.name())

        textCtrlDescription = self.getWidget('textCtrlDescription')
        textCtrlDescription.SetValue(definitionToEdit.description() or '')

        executable = self.definitionToEdit.executable()
        executableType = self.contextManager.getTypeForExecutable(executable)
        index = self.executableTypes.index(executableType)
        choiceExecutableType = self.getWidget('choiceExecutableType')
        choiceExecutableType.SetSelection(index)        
        self.displayGuiForEditingExecutableType(executableType)

        # populate parameters
        self.populateParameters()
        self.Refresh()

        return

    def OnChoiceExecutable(self, event):
        choiceExecutableType = self.getWidget('choiceExecutableType')
        executableType = choiceExecutableType.GetStringSelection()
        # type
        index = self.executableTypes.index(executableType)
        choiceExecutableType.SetSelection(index)

        self.displayGuiForEditingExecutableType(executableType)

        self.Refresh()
        return


    def displayGuiForEditingExecutableType(self, executableType):

        executable = self.definitionToEdit.executable()
        if executable is None or \
           not self.contextManager.getTypeForExecutable(executable) == executableType:
            executableClass = self.contextManager.getClassForExecutableType(executableType)
            executable = executableClass()

        dialogSizer = self.GetSizer()
        item =dialogSizer.GetItem(1)
        gbSizer = item.GetSizer()

        if executableType == 'Shell':
            # turn off unnecessary widgets and
            # turn on necessary widgets

            for x, y, showShow in [(1,0,True),(1,1,True),
                                   (2,0,True),
                                   (3,0,False),(3,1,False),
                                   (4,0,False),(4,1,False),
                                   (5,0,True),(5,1,True)]:
                item = gbSizer.FindItemAtPosition(wx.GBPosition(x, y))
                item.Show(showShow)

            for widgetId, value in [
                ('textCtrlExecutablePath', ' '.join(executable.path())),
                ('checkBoxStageable', executable.stageable()),
                ('textCtrlStaticArgs', ' '.join(executable.staticArgs()))
                ]:
                widget = self.getWidget(widgetId)
                widget.SetValue(value)

            pass
        elif executableType == 'Hadoop Jar':
            # turn off unnecessary widgets
            # turn on necessary widgets

            for x, y, showShow in [(1,0,False),(1,1,False),
                                   (2,0,False),
                                   (3,0,True),(3,1,True),
                                   (4,0,True),(4,1,True),
                                   (5,0,False),(5,1,False)]:
                item = gbSizer.FindItemAtPosition(wx.GBPosition(x, y))
                item.Show(showShow)

            for widgetId, value in [
                ('textCtrlJarFile', ' '.join(executable.jarFile())),
                ('textCtrlJarClass', ' '.join(executable.jarClass())),
                ('textCtrlStaticArgs', ' '.join(executable.staticArgs()))
                ]:
                widget = self.getWidget(widgetId)
                widget.SetValue(value)

            pass
        elif executableType == 'Hadoop Streaming':
            # turn off unnecessary widgets
            # turn on necessary widgets

            for x, y, showShow in [(1,0,False),(1,1,False),
                                   (2,0,False),
                                   (3,0,False),(3,1,False),
                                   (4,0,False),(4,1,False),
                                   (5,0,False),(5,1,False)]:
                item = gbSizer.FindItemAtPosition(wx.GBPosition(x, y))
                item.Show(showShow)
            pass

        elif executableType == 'Hadoop Pipes':
            # turn off unnecessary widgets
            # turn on necessary widgets

            for x, y, showShow in [(1,0,True),(1,1,True),
                                   (2,0,False),
                                   (3,0,False),(3,1,False),
                                   (4,0,False),(4,1,False),
                                   (5,0,False),(5,1,False)]:
                item = gbSizer.FindItemAtPosition(wx.GBPosition(x, y))
                item.Show(showShow)
            for widgetId, value in [
                ('textCtrlExecutablePath', ' '.join(executable.pipesFile())),
                ]:
                widget = self.getWidget(widgetId)
                widget.SetValue(value)

            pass

        gbSizer.Layout()
        self.Layout()

        return

    def populateParameters(self):

        self.listBoxParameters.Clear()

        parametersTable = self.definitionToEdit.parametersTable()

        filter = RelationalModule.ColumnValueFilter(
            'parameter',
            FilterModule.ObjectKeyMatchesFilter(
                filter = FilterModule.EquivalenceFilter(ParameterModule.PORT_TYPE_DATA),
                keyFunction=lambda x: x.portType()
            )
        )

        parameters = RelationalModule.Table.reduceRetrieve(
            parametersTable,
            filter,
            ['parameter'],
            []
        )
        parameterNames = [x.name() for x in parameters]


        self.listValueMap = dict(zip(parameterNames, parameters))
        self.listKeys = parameterNames

        map(self.listBoxParameters.Append, self.listKeys)

        return

    def getSelectedParameter(self):

        # this will return the selectedIndex
        selectionIndices = self.listBoxParameters.GetSelections()
        if len(selectionIndices) is not 1:
            raise ValueError('need to select a parameter')
        selectedLabel = self.listKeys[selectionIndices[0]]
        parameter = self.listValueMap[selectedLabel]
        return parameter


    def saveValues(self):

        definitionToEdit = self.definitionToEdit

        hasBeenModified = False
        
        textCtrlName = self.getWidget('textCtrlName')
        if not textCtrlName.GetValue() == definitionToEdit.name():
            definitionToEdit.name(textCtrlName.GetValue())
            hasBeenModified = True

        textCtrlDescription = self.getWidget('textCtrlDescription')
        if not textCtrlDescription.GetValue() == definitionToEdit.description():
            definitionToEdit.description(textCtrlDescription.GetValue())
            hasBeenModified = True

        choiceExecutableType = self.getWidget('choiceExecutableType')
        executableType = choiceExecutableType.GetStringSelection()

        executable = definitionToEdit.executable()
        currentExecutableType = self.contextManager.getTypeForExecutable(executable)
        if not currentExecutableType == executableType:
            executableClass = self.contextManager.getClassForExecutableType(executableType)
            executable = executableClass()
            definitionToEdit.executable(executable)
            hasBeenModified = True

        if executableType == 'Shell':
            textCtrlExecutablePath = self.getWidget('textCtrlExecutablePath')
            executablePath = textCtrlExecutablePath.GetValue().split(' ')
            if not executable.path() == executablePath:
                executable.path(executablePath)
                hasBeenModified = True

            checkBoxStageable = self.getWidget('checkBoxStageable')
            isStageable = checkBoxStageable.GetValue()
            if not executable.stageable() == isStageable:
                executable.stageable(isStageable)
                hasBeenModified = True

            textCtrlStaticArgs = self.getWidget('textCtrlStaticArgs')
            staticArgs = textCtrlStaticArgs.GetValue().split(' ')
            if not executable.staticArgs() == staticArgs:
                executable.staticArgs(staticArgs)
                hasBeenModified = True

        elif executableType == 'Hadoop Jar':

            textCtrlJarFile = self.getWidget('textCtrlJarFile')
            jarFile = textCtrlJarFile.GetValue().split(' ')
            if not executable.jarFile() == jarFile:
                executable.jarFile(jarFile)
                hasBeenModified = True

            textCtrlJarClass = self.getWidget('textCtrlJarClass')
            jarClass = textCtrlJarClass.GetValue().split(' ')
            if not executable.jarClass() == jarClass:
                executable.jarClass(jarClass)
                hasBeenModified = True
            pass

        elif executableType == 'Hadoop Streaming':
            executable.jarFile([self.contextManager.getHadoopStreamingJar()])
            executable.jarClass([])
            pass

        elif executableType == 'Hadoop Pipes':
            textCtrlExecutablePath = self.getWidget('textCtrlExecutablePath')
            pipesFile = textCtrlExecutablePath.GetValue().split(' ')
            if not executable.pipesFile() == pipesFile:
                executable.pipesFile(pipesFile)
                hasBeenModified = True
            pass

        if hasBeenModified:
            self.contextToEdit.isModified(True)
            import pomsets_app.gui.event as EventModule
            event = EventModule.DefinitionValuesChangedEvent(
                definition=definitionToEdit)
            self.contextManager.app().GetTopWindow().AddPendingEvent(event)
        
        return


    def OnButtonOk(self, event):
        self.saveValues()
        WidgetModule.Dialog.OnButtonOk(self, event)
        return

    def OnButtonApply(self, event):
        self.saveValues()
        return

    def OnAddParameter(self, event):

        name = '_'.join(['parameter', uuid.uuid4().hex[:3]])

        # default to input parameter
        parameter = ParameterModule.DataParameter(
            name=name, 
            portDirection=ParameterModule.PORT_DIRECTION_INPUT
        )

        self.definitionToEdit.addParameter(parameter)
        self.populateParameters()
        return


    def OnEditParameter(self, event):
        # contextManager, window, parameterToEdit
        import pomsets_app.gui.utils as GuiUtilsModule
        parameterToEdit = self.getSelectedParameter()
        GuiUtilsModule.editParameter(self.contextManager, self, parameterToEdit)
        self.populateParameters()
        return


    def OnRemoveParameter(self, event):
        import pomsets_app.gui.utils as GuiUtilsModule
        parameterToEdit = self.getSelectedParameter()
        self.definitionToEdit.removeParameter(parameterToEdit)
        return

    # END class EditDefinitionDialog
    pass


class EditDefinitionParameterDialog(WidgetModule.Dialog):

    XRC_NAME = 'EditDefinitionParameterDialog'
    WIDGET_BINDINGS = [
        ('wxID_OK', 'OnButtonOk', wx.EVT_BUTTON),
        ('wxID_APPLY', 'OnButtonApply', wx.EVT_BUTTON),
        ('checkBoxList', 'OnCheckboxList', wx.EVT_CHECKBOX),
        ('checkBoxCommandline', 'OnCheckboxCommandline', wx.EVT_CHECKBOX)
    ]

    def __init__(self, xmlResource, parent=None):
        WidgetModule.Dialog.__init__(
            self, 
            xmlResource.LoadDialog(
                parent, EditDefinitionParameterDialog.XRC_NAME))
        self.Fit()
        self.SetAutoLayout(True)

        self.parameterToEdit = None
        return


    def setVisibilityOfWidgetItems(self):

        checkboxList = self.getWidget('checkBoxList')
        checkboxCommandline = self.getWidget('checkBoxCommandline')

        isList = checkboxList.GetValue()
        isCommandline = checkboxCommandline.GetValue()

        dialogSizer = self.GetSizer()
        item =dialogSizer.GetItem(0)
        gbSizer = item.GetSizer()

        shouldShowFlag = True
        shouldShowDistributeFlag = True
        # enable the flag items only if commandline is checked
        if isCommandline:
            shouldShowFlag = True
            # enable the distribute flag items only if 
            # both commandline and list are checked
            if isList:
                shouldShowDistributeFlag = True
            else:
                shouldShowDistributeFlag = False
            pass
        else:
            shouldShowFlag = False
            shouldShowDistributeFlag = False
            pass

        visibilitySettings = [
            (4,0,shouldShowFlag),
            (4,1,shouldShowFlag),
            (8,0,shouldShowDistributeFlag)
            ]
        for x, y, showShow in visibilitySettings:
            item = gbSizer.FindItemAtPosition(wx.GBPosition(x, y))
            item.Show(showShow)

        return


    def OnCheckboxList(self, event):
        self.setVisibilityOfWidgetItems()
        return

    def OnCheckboxCommandline(self, event):
        self.setVisibilityOfWidgetItems()
        return


    def getPortDirections(self):
        return ['Input', 'Output']

    def getIndexForPortDirection(self, direction):
        if direction == ParameterModule.PORT_DIRECTION_INPUT:
            return 0
        if direction == ParameterModule.PORT_DIRECTION_OUTPUT:
            return 1
        raise NotImplementedError('not implemented for direction %s' % direction)


    def populate(self):

        choiceDirection = self.getWidget('choiceDirection')
        map(choiceDirection.Append,
            ['Input', 'Output'])

        return

    def setObjectToEdit(self, parameterToEdit):

        self.parameterToEdit = parameterToEdit        

        textCtrlName = self.getWidget('textCtrlName')
        textCtrlName.SetValue(parameterToEdit.name())

        textCtrlDescription = self.getWidget('textCtrlDescription')
        description = parameterToEdit.description() or ''
        textCtrlDescription.SetValue(description)


        choiceDirection = self.getWidget('choiceDirection')
        parameterDirection = parameterToEdit.portDirection()
        choiceDirection.SetSelection(
            self.getIndexForPortDirection(parameterDirection))


        checkBoxCommandline = self.getWidget('checkBoxCommandline')
        isCommandline = parameterToEdit.getAttribute(
            ParameterModule.PORT_ATTRIBUTE_COMMANDLINE)
        checkBoxCommandline.SetValue(isCommandline)
        if isCommandline:

            commandlineOptions = parameterToEdit.getAttribute(
                ParameterModule.PORT_ATTRIBUTE_COMMANDLINE_OPTIONS,
                defaultValue = {}
            )

            textCtrlFlag = self.getWidget('textCtrlFlag')
            prefixFlag = commandlineOptions.get(
                ParameterModule.COMMANDLINE_PREFIX_FLAG, []
            )
            textCtrlFlag.SetValue(' '.join(prefixFlag))
            pass


        checkBoxOptional = self.getWidget('checkBoxOptional')
        isOptional = parameterToEdit.getAttribute(
            ParameterModule.PORT_ATTRIBUTE_ISOPTIONAL)
        checkBoxOptional.SetValue(isOptional)

        checkBoxFile = self.getWidget('checkBoxFile')
        isFile =  parameterToEdit.getAttribute(
            ParameterModule.PORT_ATTRIBUTE_ISINPUTFILE)
        checkBoxFile.SetValue(isFile)

        checkBoxList = self.getWidget('checkBoxList')
        argumentValueIsList = parameterToEdit.getAttribute(
            ParameterModule.PORT_ATTRIBUTE_ISLIST)
        checkBoxList.SetValue(argumentValueIsList)

        if isCommandline and argumentValueIsList:
            checkBoxDistributeFlag = self.getWidget('checkBoxDistributeFlag')
            shouldDistributePrefixFlag = commandlineOptions.get(
                ParameterModule.COMMANDLINE_PREFIX_FLAG_DISTRIBUTE, False)
            checkBoxDistributeFlag.SetValue(shouldDistributePrefixFlag)
            pass

        self.setVisibilityOfWidgetItems()

        return


    def saveValues(self):

        parameter = self.parameterToEdit

        textCtrlName = self.getWidget('textCtrlName')
        parameter.name(textCtrlName.GetValue())

        textCtrlDescription = self.getWidget('textCtrlDescription')
        parameter.description(textCtrlDescription.GetValue())

        choiceDirection = self.getWidget('choiceDirection')
        choice = choiceDirection.GetStringSelection()
        if choice == 'Input':
            parameter.portDirection('input')
        elif choice == 'Output':
            parameter.portDirection('output')

        checkBoxOptional = self.getWidget('checkBoxOptional')
        parameter.setAttribute(
            ParameterModule.PORT_ATTRIBUTE_ISOPTIONAL, 
            checkBoxOptional.GetValue())

        checkBoxFile = self.getWidget('checkBoxFile')
        parameter.setAttribute(
            ParameterModule.PORT_ATTRIBUTE_ISINPUTFILE, 
            checkBoxFile.GetValue())

        checkBoxList = self.getWidget('checkBoxList')
        parameter.setAttribute(
            ParameterModule.PORT_ATTRIBUTE_ISLIST, 
            checkBoxList.GetValue())

        checkBoxCommandline = self.getWidget('checkBoxCommandline')
        parameter.setAttribute(
            ParameterModule.PORT_ATTRIBUTE_COMMANDLINE, 
            checkBoxCommandline.GetValue())


        commandlineOptions = parameter.getAttribute(
            ParameterModule.PORT_ATTRIBUTE_COMMANDLINE_OPTIONS,
            defaultValue = {}
        )

        # flag
        # distribute flag
        textCtrlFlag = self.getWidget('textCtrlFlag')
        prefixFlag = textCtrlFlag.GetValue()
        commandlineOptions[ParameterModule.COMMANDLINE_PREFIX_FLAG] = prefixFlag.split(' ')

        checkBoxDistributeFlag = self.getWidget('checkBoxDistributeFlag')
        commandlineOptions[ParameterModule.COMMANDLINE_PREFIX_FLAG_DISTRIBUTE]  = checkBoxDistributeFlag.GetValue()

        parameter.setAttribute(
            ParameterModule.PORT_ATTRIBUTE_COMMANDLINE_OPTIONS,
            commandlineOptions
        )

        return


    def OnButtonOk(self, event):
        self.saveValues()
        WidgetModule.Dialog.OnButtonOk(self, event)
        return

    def OnButtonApply(self, event):
        self.saveValues()
        return

    # END class EditDefinitionParameterDialog
    pass


class EditReferenceDefinitionDialog(WidgetModule.Dialog):
    """
    Internally we call this the EditDefinitionReferenceDialog
    but to the user, it's the Task editing dialog.
    The difference lies in that internally, 
    a task is not actually created until a pomset is executed
    but users do not usually make that kind of distinction
    """
    
    XRC_NAME = 'EditReferenceDefinitionDialog'
    WIDGET_BINDINGS = [
        ('wxID_OK', 'OnButtonOk', wx.EVT_BUTTON),
        ('wxID_APPLY', 'OnButtonApply', wx.EVT_BUTTON),
        ('choiceParameter', 'OnChoiceParameter', wx.EVT_CHOICE)
    ]

    def __init__(self, xmlResource, parent=None):
        WidgetModule.Dialog.__init__(
            self, 
            xmlResource.LoadDialog(
                parent, EditReferenceDefinitionDialog.XRC_NAME))
        self.Fit()
        self.SetAutoLayout(True)
        self.definitionToEdit = None
        
        textCtrlParameterValues = self.getWidget('textCtrlParameterValues')

        textCtrlParameterValues.SetMinSize(wx.Size(250, 100))
        textCtrlParameterValues.SetSize(wx.Size(250, 100))
        
        return

    
    def setObjectToEdit(self, definitionToEdit):
        
        self.definitionToEdit = definitionToEdit

        textCtrlName = self.getWidget('textCtrlName')
        textCtrlName.SetValue(definitionToEdit.name())

        textCtrlComment = self.getWidget('textCtrlComment')
        textCtrlComment.SetValue(definitionToEdit.comment() or '')
        
        # populate parameters
        self.populateParameters()
        self.populateParameterValues()
        self.Refresh()

        return
    
        
    def populateParameters(self):
        
        # get the parameters from the referenced definition
        
        filter = FilterModule.ObjectKeyMatchesFilter(
            keyFunction=lambda x: x.portType(),
            filter=FilterModule.EquivalenceFilter(ParameterModule.PORT_TYPE_TEMPORAL)
        )
        notFilter = FilterModule.constructNotFilter()
        notFilter.addFilter(filter)

        
        parameters = self.definitionToEdit.getParametersByFilter(notFilter)
        choiceParameter =self.getWidget('choiceParameter')
        if len(parameters) is 0:
            choiceParameter.Append('No parameters defined', None)
        else:
            for parameter in parameters:
                choiceParameter.Append(parameter.id(), parameter)
            self.selectParameter(parameters[0].id())
        return
        
    
    def OnChoiceParameter(self, event):
        self.populateParameterValues()
        self.Refresh()
        return
    
    
    def selectParameter(self, parameterId):
        choiceParameter =self.getWidget('choiceParameter')        
        item = choiceParameter.FindString(parameterId)
        choiceParameter.SetSelection(item)
        return
    
    
    def populateParameterValues(self):
        # get the parameter values from self
        choiceParameter =self.getWidget('choiceParameter')        
        selectedParameterIndex = choiceParameter.GetSelection()
        if selectedParameterIndex == wx.NOT_FOUND:
            return
        parameter = choiceParameter.GetClientData(selectedParameterIndex)
        if parameter is None:
            return
        
        parameterId = parameter.id()

        
        checkBoxStaging = self.getWidget('checkBoxStaging')
        checkBoxStaging.SetValue(
            self.definitionToEdit.parameterStagingRequired(parameterId))
                
        checkBoxSweep = self.getWidget('checkBoxSweep')
        checkBoxSweep.SetValue(
            self.definitionToEdit.isParameterSweep(parameterId))
        
        
        (dataNode, parameterToEdit) = \
         self.definitionToEdit.getParameterToEdit(parameterId)
        
        parameterId = parameterToEdit.id()
        
        values = ''
        try:
            if dataNode.hasParameterBinding(parameterId):
                values = '\n'.join(
                    dataNode.getParameterBinding(parameterId))
            else:
                logging.debug("data node has not parameter bindings for %s" % parameterId)
            pass
        except KeyError, e:
            logging.error("error on populating parameter values >> %s" % e)
            raise
        logging.debug("retrieved bindings for parameter %s >> %s" % (parameterId, values))
        
        textCtrl = self.getWidget('textCtrlParameterValues')
        textCtrl.SetValue(values)
        

        
        return
    
    
    def saveValues(self):
        print "%s saveValues" % self.__class__

        textCtrlName = self.getWidget('textCtrlName')
        textCtrlComment = self.getWidget('textCtrlComment')

        if not self.definitionToEdit.name() == textCtrlName.GetValue():
            self.definitionToEdit.name(textCtrlName.GetValue())

            import pomsets_app.gui.event as EventModule
            event = EventModule.NodeRenamedEvent(
                node = self.definitionToEdit
            )
            self.contextManager.postEvent(event)
            pass

        self.definitionToEdit.comment(textCtrlComment.GetValue())
        
        self.saveParameterValues()
        return

    
    def saveParameterValues(self):
        choiceParameter = self.getWidget('choiceParameter')
        
        selectedParameterIndex = choiceParameter.GetSelection()
        if selectedParameterIndex == wx.NOT_FOUND:
            return
        parameter = choiceParameter.GetClientData(selectedParameterIndex)
        if parameter is None:
            return
        
        parameterId = parameter.id()
        
        checkBoxStaging = self.getWidget('checkBoxStaging')
        checkBoxSweep = self.getWidget('checkBoxSweep')

        # set the value for file staging
        value = checkBoxStaging.GetValue()
        self.definitionToEdit.parameterStagingRequired(
            parameterId, value)

        # set the value for parameter sweep
        value = checkBoxSweep.GetValue()
        # need to set the value only if it has changed
        if not self.definitionToEdit.isParameterSweep(parameterId) == value:
            self.definitionToEdit.isParameterSweep(
                parameterId, value
            )
            
            # if the value has changed
            # we should send an event
            # so that we can update the node image if necessary
            # to hande for displaying parameter sweep
            import pomsets_app.gui.event as EventModule
            event = EventModule.ParameterModifiedEvent(
                definition=self.definitionToEdit,
                parameter=parameterId
            )
            self.contextManager.postEvent(event)
            pass
        
        
        (dataNode, parameterToEdit) = \
         self.definitionToEdit.getParameterToEdit(parameterId)
        parameterId = parameterToEdit.id()
        
        textCtrlParameterValues = self.getWidget('textCtrlParameterValues')
        text = textCtrlParameterValues.GetValue()
        values = text.split("\n")
        
        logging.debug('binding parameter %s to value "%s" on dataNode %s' % 
                      (parameterId, values, dataNode))
        dataNode.setParameterBinding(
            parameterId, values)
        
        
        
        return
    
    
    def OnButtonOk(self, event):
        self.saveValues()
        WidgetModule.Dialog.OnButtonOk(self, event)
        return

    def OnButtonApply(self, event):
        self.saveValues()
        return
    
    
    pass




import copy
import functools
import logging
import os
import Queue
import threading
import threadpool
import urlparse


import pomsets.resource as ResourceModule

KEY_CONTROLLER_ACCESS_KEY = 'controllerAccessKey'
KEY_CONTROLLER_SECRET_KEY = 'controllerSecretKey'
KEY_CONTROLLER_SERVICE_NAME = 'controllerServiceName'

def parseUrl(url):
    
    result = urlparse.urlparse(url)
    host = result.hostname
    port = result.port
    path = result[2]
    isSecure = (result[0] == 'https')
    
    return {
        'isSecure':isSecure,
        'host':host,
        'port':port,
        'path':path
    }


def defaultEC2Values():

    controllerAccessKey = os.getenv('EC2_ACCESS_KEY')
    controllerSecretKey = os.getenv('EC2_SECRET_KEY')
    url = os.getenv('EC2_URL')

    values = parseUrl(url)
    values.update(
        {
            KEY_CONTROLLER_ACCESS_KEY:controllerAccessKey, 
            KEY_CONTROLLER_SECRET_KEY:controllerSecretKey,
            KEY_CONTROLLER_SERVICE_NAME:'eucalyptus'
        }
    )
    return values
    



class Thread(threading.Thread):

    def __init__(self, function):
        threading.Thread.__init__(self)
        self._function = function
        return
        
    def run(self):
        self._function()
        return
    
    # END class Thread
    pass




from __future__ import with_statement

import os
import simplejson as ConfigModule
import sys
import unittest
import user

import pomsets_app.controller.automaton as AutomatonModule
import pomsets_app.utils as AppUtilsModule

class TestParse(unittest.TestCase):

    def testParseCloudCredentials(self):

	with open('test/config1') as f:

	    cfg = ConfigModule.load(f)

	    credentials = [x for x in cfg['cloud controller credentials']]
	    self.assertTrue(len(credentials) is 1)
	    
	    credential = credentials[0]
	    
	    expected = 'Eucalyptus'
	    actual = credential['service name']
	    self.assertEquals(expected, actual)
	    
	    expected = 'euca2ools'
	    actual = credential['service API']
	    self.assertEquals(expected, actual)
	    
	    values = credential['values']

	    expected = '00:00:00:00:00:00:00:00:00:00:00:00:00:00:00:00:00:00:00:00'
	    actual = values['user key pair']
	    self.assertEquals(expected, actual)

	    expected = 'pathToIdentityFile'
	    actual = values['identity file']
	    self.assertEquals(expected, actual)
	    
	    expected ='image-id'
	    actual = values['default image']
	    self.assertEquals(expected, actual)
	    
	    pass

	return
    
    
    def testParseRemoteExecuteCredentials(self):
	
	with open('test/config1') as f:

	    cfg = ConfigModule.load(f)

	    credentials = [x for x in cfg['remote execute credentials']]
	    self.assertTrue(len(credentials) is 1)
	    
	    credential = credentials[0]
	    
	    expected = 'remotehost'
	    actual = credential['hostname']
	    self.assertEquals(expected, actual)
	    
	    expected = 'myself'
	    actual = credential['user']
	    self.assertEquals(expected, actual)
	    
	    expected = 'ssh/key/file'
	    actual = credential['keyfile']
	    self.assertEquals(expected, actual)
	    
	    pass
	
	return
    
    
    def testParseHadoopConfigurations(self):
	with open('test/config1') as f:

	    cfg = ConfigModule.load(f)

	    configurations = [x for x in cfg['hadoop configurations']]
	    self.assertTrue(len(configurations) is 1)
	    
	    configuration = configurations[0]
	    
	    expected = 'localhost'
	    actual = configuration['hostname']
	    self.assertEquals(expected, actual)
	    
	    expected = '/hadoop/core'
	    actual = configuration['home']
	    self.assertEquals(expected, actual)
	    
	    pass
	
	return


    def testParseDotCommandPath(self):
	with open('test/config1') as f:
	    cfg = ConfigModule.load(f)

	    configurations = [x for x in cfg['graphviz configurations']]
	    self.assertTrue(len(configurations) is 1)

            configuration = configurations[0]
            expected = 'localhost'
            actual = configuration['hostname']

            expected = '/path/to/dot'
            actual = configuration['dot command path']
            pass

        return

    """
    def testParseLibraryDefinition(self):
	with open('%s/app/src/test/config1' % APP_ROOT) as f:

	    cfg = ConfigModule.load(f)

	    libraryLocations = [x for x in cfg['library definitions']]
	    self.assertTrue(len(libraryLocations) is 1)

	    expected = ['/Users/myself/.pomsets/library']
	    actual = libraryLocations
	    self.assertEquals(expected, actual)
	    pass
	    
	return
    """
    
    # END class TestParse
    pass


class TestSave(unittest.TestCase):

    PATH_ORIGINAL = 'test/config1' 
    PATH_SAVE = '/tmp/foo'
    
    def setUp(self):
        config = AppUtilsModule.loadConfig(configPath=TestSave.PATH_ORIGINAL)

	automaton = AutomatonModule.Automaton()
	automaton.loadConfig(config)
	
	self.automaton = automaton
	return
    
    def tearDown(self):
	if os.path.exists(TestSave.PATH_SAVE):
	    os.unlink(TestSave.PATH_SAVE)
	    
	return
    
    def testSaveConfig1(self):
	"""
	need to ensure that no values get corrupted
	just save out, with no modifications
	"""
	
        # save out to file
        configData = AppUtilsModule.createNewConfigDataObject()
        self.automaton.addConfigToSave(configData)
        AppUtilsModule.saveConfig(configData, configPath=TestSave.PATH_SAVE)


	originalCfg = None
	with open(TestSave.PATH_ORIGINAL) as f:
	    originalCfg = ConfigModule.load(f)
	
	savedCfg = None
	with open(TestSave.PATH_SAVE) as f:
	    savedCfg = ConfigModule.load(f)
	    
	self.assertTrue(originalCfg is not None)
	self.assertTrue(savedCfg is not None)
	
	self.assertEquals(set(originalCfg.keys()), set(savedCfg.keys()))

        for key in originalCfg.keys():
            self.assertEquals(originalCfg[key], savedCfg[key])
	
	
	return
    
    def testSaveConfig2(self):
	"""
	need to ensure that changes get saved out properly
	change some values
	assert that the values got changed
	"""
	
	return
    
    # END class TestSave
    pass




def main():

    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestParse, 'test'))
    suite.addTest(unittest.makeSuite(TestSave, 'test'))
    suite.addTest(unittest.makeSuite(TestAutomaton, 'test'))

    runner = unittest.TextTestRunner()
    runner.run(suite)
    return

if __name__=="__main__":
    main()




import wx
import wx.lib.agw.foldpanelbar

import zgl_graphdrawer.Application as ApplicationModule
import zgl_graphdrawer.Frame as zglFrameModule

import pomsets_app.controller.automaton as AutomatonModule
import pomsets_app.gui.event as EventModule
import pomsets_app.gui.canvas as CanvasModule
import pomsets_app.gui.menu as MenuModule

import pomsets.definition as DefinitionModule
import pomsets.library as LibraryModule

import pomsets_app.gui.frame as FrameModule

class WxFrame(zglFrameModule.wxFrame, FrameModule.Frame):

    FBP_MIN_WIDTH = 200
    
    def __init__(self, app, *args, **kwds):
        zglFrameModule.wxFrame.__init__(self, *args, **kwds)
        self.app(app)

        self.layoutContent()
        self.Centre()
        
        self.bindEvents()
        
        pass

    def displayPomset(self, pomsetContext, pomset=None):
        FrameModule.Frame.displayPomset(self, pomsetContext, pomset=pomset)

        # need to call this to force the canvas to update
        self.resizeCanvas()

        # update the fold panel menu
        # TODO: each of these should be able to update
        #       when events are fired instead
        self.populateActivePomsetTreeCtrl()
        self.populateLibraryDefinitionTreeCtrl()
        self.populateLoadedDefinitionsPane()

        self.updatePomsetMenu()
        return

    def bindEvents(self):
        zglFrameModule.wxFrame.bindEvents(self)

        self.Bind(wx.EVT_SPLITTER_SASH_POS_CHANGING, self.OnSplitterReposition)

        self.currentActivePomsetTreeCtrl.Bind(
            wx.EVT_LEFT_DOWN, self.OnFbpLeftMouseDown)
        self.currentActivePomsetTreeCtrl.Bind(
            wx.EVT_LEFT_DCLICK, self.OnFbpLeftMouseDoubleClick)
        self.currentActivePomsetTreeCtrl.Bind(
            wx.EVT_RIGHT_DOWN, self.OnActivePomsetsPaneRightMouseDown)
        
        self.libraryDefinitionTreeCtrl.Bind(
            wx.EVT_TREE_BEGIN_DRAG, self.OnBeginDragDefinitionTreeCtrl)
        self.libraryDefinitionTreeCtrl.Bind(
            wx.EVT_LEFT_DOWN, self.OnFbpLeftMouseDown)
        self.libraryDefinitionTreeCtrl.Bind(
            wx.EVT_RIGHT_DOWN, self.OnLibraryDefinitionPaneRightMouseDown)
        
        self.loadedDefinitionTreeCtrl.Bind(
            wx.EVT_TREE_BEGIN_DRAG, self.OnBeginDragDefinitionTreeCtrl)
        self.loadedDefinitionTreeCtrl.Bind(
            wx.EVT_LEFT_DOWN, self.OnFbpLeftMouseDown)
        self.loadedDefinitionTreeCtrl.Bind(
            wx.EVT_RIGHT_DOWN, self.OnLoadedDefinitionPaneRightMouseDown)
        
        self.Bind(EventModule.EVT_DEFINITION_VALUES_CHANGED, 
                  self.OnDefinitionValuesChanged)

        import platform
        if platform.system() in ['Darwin']:
            print "calling wx.App.SetMacExitMenuItemId(wx.ID_EXIT)"
            wx.App.SetMacExitMenuItemId(wx.ID_EXIT)
        self.Bind(wx.EVT_CLOSE, self.OnExitApplication)
        return
    
    def OnExitApplication(self, event):

        # first check whether there are any modified pomsets
        # if so, query the user as to whether 
        contextManager = self.app().contextManager()

        shouldQueryUser = any(
            [x.isModified() 
             for x in contextManager.activePomsetContexts()])

        import sys
        shouldExit = True
        if shouldQueryUser:
            shouldExit = False

            resourcePath = contextManager.resourcePath()
            res =  wx.xrc.XmlResource(
                os.path.join(resourcePath, 'xrc', 'verify exit application.xrc'))

            import pomsets_app.gui.wx.widget.application as WidgetModule
            dialog = WidgetModule.VerifyExitDialog(res, self)

            try:
                if dialog.ShowModal() == wx.ID_OK:
                    shouldExit = True
                    pass
                pass
            except Exception, e:
                # if any errors, should exit
                sys.exit(1)
                raise
            finally:
                dialog.Destroy()
            pass


        if shouldExit:
            sys.exit(0)

        return
    

    
    
    def layoutContent(self):
        self.createApplicationMenus()
        
        splitter = wx.SplitterWindow(self, -1)
        self.splitter = splitter
        
        self._updateActivePomsetTreeCtrlSelection = True

        foldPanelBar = self.initializeFoldPanel(splitter)
        canvas = self.initializeCanvas(splitter)
        
        # need to set this so that we can update the canvas 
        # for the callbacks later on
        self.canvas = canvas
        canvas.app = lambda: self.app()

        splitter.SplitVertically(foldPanelBar, canvas, Frame.FBP_MIN_WIDTH)
 
        pass

    
    def initializeCanvas(self, parent):
        canvas = CanvasModule.Canvas(parent, 
                                  ApplicationModule.DEFAULT_WIDTH,
                                  ApplicationModule.DEFAULT_HEIGHT,
                                  style=wx.FULL_REPAINT_ON_RESIZE)
        canvas.resetDrawables()
        return canvas
    
    
    def initializeFoldPanel(self, parent):
        
        foldPanelBar = wx.lib.agw.foldpanelbar.FoldPanelBar(parent, -1)
        
        item = foldPanelBar.AddFoldPanel("Active pomsets", True)
        currentActivePomsetTreeCtrl = wx.TreeCtrl(
            item, wx.ID_ANY, style=wx.TR_HIDE_ROOT|wx.TR_SINGLE|wx.TR_HAS_BUTTONS)
        foldPanelBar.AddFoldPanelWindow(
            item, currentActivePomsetTreeCtrl)
        self.currentActivePomsetTreeCtrl = currentActivePomsetTreeCtrl
        self.populateActivePomsetTreeCtrl()
        # need to do this ensure that 
        # the treectrl is drawn correctly
        foldPanelBar.Collapse(item)
        foldPanelBar.Expand(item)        
        
        item = foldPanelBar.AddFoldPanel("Library definitions", True)
        libraryDefinitionTreeCtrl = wx.TreeCtrl(
            item, wx.ID_ANY, style=wx.TR_HIDE_ROOT|wx.TR_SINGLE|wx.TR_HAS_BUTTONS)
        foldPanelBar.AddFoldPanelWindow(item, libraryDefinitionTreeCtrl)
        self.libraryDefinitionTreeCtrl = libraryDefinitionTreeCtrl
        self.populateLibraryDefinitionTreeCtrl()
        # need to do this ensure that 
        # the treectrl is drawn correctly
        foldPanelBar.Collapse(item)
        foldPanelBar.Expand(item)

        
        item = foldPanelBar.AddFoldPanel("Loaded definitions", True)
        loadedDefinitionsPane = self.buildLoadedDefinitionsPane(item)
        foldPanelBar.AddFoldPanelWindow(item, loadedDefinitionsPane)
        # need to do this ensure that 
        # the treectrl is drawn correctly
        foldPanelBar.Collapse(item)
        foldPanelBar.Expand(item)

        
        # TODO:
        # add stuff for 
        # - eucalyptus/amazon aws
        # - celery
        # - hadoop
        item = foldPanelBar.AddFoldPanel("Application configuration", True)

        return foldPanelBar

    
    def buildLoadedDefinitionsPane(self, parent):
        
        import pomsets_app.gui.widget as WidgetModule
        
        contextManager = self.app().contextManager()
        
        loadedDefinitionTreeCtrl = wx.TreeCtrl(
            parent, wx.ID_ANY, style=wx.TR_HIDE_ROOT|wx.TR_SINGLE|wx.TR_HAS_BUTTONS)
        
        self.loadedDefinitionTreeCtrl = loadedDefinitionTreeCtrl

        self.populateLoadedDefinitionsPane()
        
        return self.loadedDefinitionTreeCtrl
    
    
    
    
    def addPomsetToTreeCtrl(self, ctrl, parentItem, 
                            pomsetContext, pomset, name=None, isRoot=False):
        
        if name is None:
            name = pomset.name() or "anonymous"
        
        if isRoot and pomsetContext.isModified():
            name = name + ' (modified)'
            
        pomsetItem = ctrl.AppendItem(
            parentItem, name, data=wx.TreeItemData((pomsetContext, pomset)))
        
        if not pomset.isAtomic():
            for node in pomset.nodes():
                nodeName = "%s (def: %s)" % (
                    node.name() or "anonymous",
                    node.definitionToReference().name() or "anonymous"
                )
                
                self.addPomsetToTreeCtrl(
                    ctrl, pomsetItem, pomsetContext,
                    node.definitionToReference(),
                    name=nodeName,
                    isRoot = False
                )
            
        return
    
    def populateActivePomsetTreeCtrl(self):
        
        ctrl = self.currentActivePomsetTreeCtrl
        ctrl.DeleteAllItems()
        
        contextManager = self.app().contextManager()
        activePomsetContexts = contextManager.activePomsetContexts()
        numItems = max(len(activePomsetContexts), 1)
        root = ctrl.AddRoot('Active pomsets')
        if len(activePomsetContexts) is 0:
            ctrl.AppendItem(root, 'No active pomsets')
        else:
            for activePomsetContext in activePomsetContexts:
                activePomset = activePomsetContext.pomset()
                self.addPomsetToTreeCtrl(
                    ctrl, root, activePomsetContext, activePomset, isRoot=True)
            pass
 
        # select the currently displayed pomset
        self.selectActivePomset(contextManager.activePomset())
        
        height = min(max(numItems * 25, 100), 250)
        ctrl.SetSize(wx.Size(Frame.FBP_MIN_WIDTH, height))
        
        return
    
    def populateLibraryDefinitionTreeCtrl(self):
        ctrl = self.libraryDefinitionTreeCtrl
        contextManager = self.app().contextManager()

        ctrl.DeleteAllItems()
        root = ctrl.AddRoot('Library definitions')
        
        definitionContexts = [x for x in contextManager.getLibraryDefinitions()]
        for definitionContext in definitionContexts:
            definition = definitionContext.pomset()

            name = definition.name()
            
            # we don't want users to have access 
            # to the boostrap pomsets
            if definition.id() in [LibraryModule.ID_LOADLIBRARYDEFINITION,
                                   LibraryModule.ID_BOOTSTRAPLOADER]:
                continue
            
            ctrl.AppendItem(root, name, data=wx.TreeItemData(definitionContext))
            pass

                
        numItems = len(definitionContexts)
        height = min(max(numItems * 25, 100), 250)
        ctrl.SetSize(wx.Size(Frame.FBP_MIN_WIDTH, height))
                
        return

    
    def populateLoadedDefinitionsPane(self):
        ctrl = self.loadedDefinitionTreeCtrl
        contextManager = self.app().contextManager()

        # if something is currently selected
        # let's save that off
        selectedItem = ctrl.GetSelection()
        selectedContext = None
        if selectedItem.IsOk():
            data = ctrl.GetItemData(selectedItem)
            if data:
                selectedContext = data.GetData()
        selectedItem = None
        
        ctrl.DeleteAllItems()
        root = ctrl.AddRoot('Loaded definitions')
        
        definitionContexts = [x for x in contextManager.getLoadedDefinitions()]
        if len(definitionContexts) is 0:
            ctrl.AppendItem(root, 'No loaded definitions')
        else:
            for definitionContext in definitionContexts:
                definition = definitionContext.pomset()

                name = definition.name() or "Anonymous"
                if definitionContext.isModified():
                    name = name + ' (modified)'

                item = ctrl.AppendItem(
                    root, name, data=wx.TreeItemData(definitionContext))
                if definitionContext is selectedContext:
                    selectedItem = item
            pass

        if selectedItem is not None:
            ctrl.SelectItem(selectedItem)
        
        numItems = max(len(definitionContexts), 1)
        height = min(max(numItems * 25, 100), 250)
        ctrl.SetSize(wx.Size(Frame.FBP_MIN_WIDTH, height))
                
        return

    
    
    def Show(self, *args, **kwds):
        self.canvas._frameIsInProcessOfShowing = True
        zglFrameModule.wxFrame.Show(self, *args, **kwds)
        self.canvas._frameIsInProcessOfShowing = False
        self.resizeCanvas()
        return
    
    def moveToRefreshCanvas(self):
        """
        for some reason, the canvas only refreshes 
        its positioning relative to its containers
        only when the frame is moved
        """
        currentPosition = self.GetPosition()
        x,y = currentPosition.Get()
        point = wx.Point(x+1,y)
        self.SetPosition(point)
        self.SetPosition(currentPosition)
        return
    
    
    def resizeCanvas(self):
        """
        A lot of places will call this
        in order to force the canvas to refresh correctly
        """
        canvasWindow = self.splitter.GetWindow2()
        windowSize = canvasWindow.GetSize()
        newWidth = max(windowSize.GetWidth(), 10)
        newHeight = max(windowSize.GetHeight(), 10)
        self.canvas.resize(newWidth, newHeight)
        
        # we need to do this to force the canvas
        # to reposition itself relative to its containers
        self.moveToRefreshCanvas()

        return

    
    def OnResize(self, event):
        self.resizeCanvas()
        event.Skip()
        return
    
    
    def OnSplitterReposition(self, event):
        self.resizeCanvas()
        return
    
    
    def createApplicationMenus(self):
        menuBar = wx.MenuBar()
    
        # create a new menu for file/serialization stuff
    
        menuPomset = wx.Menu()
        menuPomset.Append(wx.ID_NEW, "New", "Create new pomset")
        menuPomset.Append(wx.ID_OPEN, "Load", "Load pomset")
        menuPomset.Append(wx.ID_CLOSE, "Close", "Close pomset")
        menuPomset.Append(wx.ID_SAVE, "Save", "Save pomset")
        menuPomset.Append(wx.ID_SAVEAS, "Save as", "Save pomset as")
        ## menuPomset.AppendSeparator()
        ## create a new menu for the pomset stuff
        ## menuPomset.Append(MenuModule.ID_DEFINITIONS_EDIT, "Edit definitions")
        ## menuPomset.Append(MenuModule.ID_NODE_ADD, "Add node")
        ## menuPomset.Append(MenuModule.ID_NODE_REMOVE, "Remove selected nodes")
        menuBar.Append(menuPomset, "&Pomset")
        
        # create a new menu for pomset stuff
        menuExecute = wx.Menu()
        
        menuExecute.Append(
            AutomatonModule.ID_EXECUTE_PREVIEW, "Preview", "Preview commands")

        menuExecute.AppendSeparator()
    
        menuExecute.Append(
            AutomatonModule.ID_EXECUTE_LOCAL, "Local", "Execute local")


        # look into configuration 
        # to determine if should add remote execute item
        contextManager = self.app().contextManager()
        automaton = contextManager.automaton()
        configuration = automaton.otherConfigurations()
        if configuration.get('should display remote execute menu item', False):
            # TODO:
            # should verify that remote execute credentials have been specified
            menuExecute.Append(
                AutomatonModule.ID_EXECUTE_REMOTE, "Remote", "Execute remote")

        menuExecute.Append(
            AutomatonModule.ID_EXECUTE_EUCA2OOLS, 
            "Euca2ools", "Execute euca2ools")
        
        menuBar.Append(menuExecute, "&Execute")
    
        # create a new menu for connection stuff
        # TODO:
        # "connect" and "disconnect" should be a menu with their own menu items
        # those menu items will represent the connections
        menuConfiguration = wx.Menu()
        
        menuConfiguration.Append(
            MenuModule.ID_ACCOUNT_MANAGE, 'VMs', 'Manage virtual machines')
        menuConfiguration.Append(
            MenuModule.ID_HADOOP_MANAGE, 'Hadoop', 'Manage Hadoop')
        menuConfiguration.Append(
            MenuModule.ID_EUCA2OOLS_MANAGE, 'Euca2ools', 'Manage euca2ools')
        menuBar.Append(menuConfiguration, "&Configuration")
    
    

        menuHelp = wx.Menu()
        menuHelp.Append(wx.ID_ABOUT, "&About",
                        "More information About this program")
        menuBar.Append(menuHelp, "&Help")

        self.SetMenuBar(menuBar)
    


        eventHandler = EventModule.MenuEventHandler(self)
        self.PushEventHandler(eventHandler)
        eventHandler.bindEvents()
    
        pass

    def OnActivePomsetsPaneRightMouseDown(self, event):
        position = event.GetPosition()
        
        item, where = self.currentActivePomsetTreeCtrl.HitTest(position)
        
        userData = None
        if item.IsOk():
            data = self.currentActivePomsetTreeCtrl.GetItemData(item)
            userData = data.GetData()
        
        # create a popup menu
        popupMenu = MenuModule.ActivePomsetContextualMenu(
            self, userData, parent=self.currentActivePomsetTreeCtrl)

        popupMenu.bindEvents()
        self.currentActivePomsetTreeCtrl.PopupMenu(
            popupMenu, position)

        # according to http://wiki.wxpython.org/PopupMenuOnRightClick
        # need to call destroy
        popupMenu.Destroy()

        return


    def OnLibraryDefinitionPaneRightMouseDown(self, event):

        position = event.GetPosition()
        
        item, where = self.libraryDefinitionTreeCtrl.HitTest(position)
        
        userData = None
        if item.IsOk():
            data = self.libraryDefinitionTreeCtrl.GetItemData(item)
            userData = data.GetData()
        
        # create a popup menu
        popupMenu = MenuModule.LibraryDefinitionContextualMenu(
            self, userData, parent=self.libraryDefinitionTreeCtrl)

        popupMenu.bindEvents()
        self.libraryDefinitionTreeCtrl.PopupMenu(
            popupMenu, position)

        # according to http://wiki.wxpython.org/PopupMenuOnRightClick
        # need to call destroy
        popupMenu.Destroy()

        return


    def OnLoadedDefinitionPaneRightMouseDown(self, event):
        
        position = event.GetPosition()
        
        item, where = self.loadedDefinitionTreeCtrl.HitTest(position)
        
        userData = None
        if item.IsOk():
            data = self.loadedDefinitionTreeCtrl.GetItemData(item)
            userData = data.GetData()
        
        # create a popup menu
        popupMenu = MenuModule.LoadedDefinitionContextualMenu(
            self, userData, parent=self.loadedDefinitionTreeCtrl)

        popupMenu.bindEvents()
        self.loadedDefinitionTreeCtrl.PopupMenu(
            popupMenu, position)

        # according to http://wiki.wxpython.org/PopupMenuOnRightClick
        # need to call destroy
        popupMenu.Destroy()

        return
    
    
    
    def OnBeginDragDefinitionTreeCtrl(self, event):

        item = event.EventObject.GetSelection()
        data = event.EventObject.GetItemData(item)
        self.dragSelection = data.GetData()
        
        event.Skip()
        return
    
    
    def OnEndDragDefinitionTreeCtrl(self, event):
        print "on end drag treectrl"
        return

    
    def OnFbpLeftMouseDoubleClick(self, event):

        item = event.EventObject.GetSelection()
        data = event.EventObject.GetItemData(item)
        
        pomsetData = data.GetData()
        if pomsetData is None:
            # this happens when the user double clicks
            # multiple times in rapid succession
            # and the UI has not has a chance to attach
            # the data model yet
            return
        pomsetContext, pomset = pomsetData

        # TODO:
        # currently only handles root
        # this should handle the case that 
        # a non-root node is referencing a composite definition
        # should show in that case as well
        # (also remember to disable editing of the referenced child
        #  is a library definition)
        if not isinstance(pomset, DefinitionModule.CompositeDefinition):
            return
        
        # because this is activated as s result of
        # chaing the selection on the tree ctrl
        # we want to disable the updating of that selection
        # and avoid an infinite loop
        self._updateActivePomsetTreeCtrlSelection = False
        contextManager = self.app().contextManager()
        contextManager.activePomsetContext(pomsetContext)
        self.displayPomset(pomsetContext, pomset)
        self._updateActivePomsetTreeCtrlSelection = True

        return
    
    def OnFbpLeftMouseUp(self, event):
        return
    
    def OnFbpLeftMouseDown(self, event):
        if hasattr(self, 'dragSelection'):
            delattr(self, 'dragSelection')
        event.Skip()
        return
    
    
    def OnDefinitionValuesChanged(self, event):
        
        # handle the loaded definitions pane,
        # in the case the name got changed
        self.populateLoadedDefinitionsPane()
        
        return

    def processDragSelection(self, event):

        # add node only if the pomset has not been executed
        pomsetContext, pomset = self._currentPomsetInfo
        contextManager = self.app().contextManager()
        if not contextManager.hasTaskForPomset(pomsetContext.pomset()):
            # create the node
            import pomsets_app.gui.utils as GuiUtilsModule
            import uuid
            definitionContext = self.dragSelection
            definition = definitionContext.pomset()
            name = '%s_%s' % (definition.name() or "anonymous",
                              uuid.uuid4().hex[:3])
            position = event.GetPosition()

            contextManager = self.app().contextManager()
            GuiUtilsModule.addNode(
                contextManager, definition, name, position)

        # remove the attribute
        delattr(self, 'dragSelection')
        return


    def setMenuItemActive(self, menuTitle, menuItemTitle, isActive):
        menuBar = self.GetMenuBar()
        menuItem = menuBar.FindMenuItem(menuTitle, menuItemTitle)
        # menuItem.Enable(isActive)
        menuBar.Enable(menuItem, isActive)
        return
    
    # END class WxFrame
    pass

